#!/usr/bin/php
<?php

class HttpClient
{
	protected $curl=null;
	protected $options=null;
	protected $url=null;

	public function __construct($base_url,$cookies=FALSE){
		$this->url = $base_url;

		$options = array();
		$options[CURLOPT_FOLLOWLOCATION]=TRUE;
		$options[CURLOPT_RETURNTRANSFER]=TRUE;
		$options[CURLOPT_NOSIGNAL]=TRUE;
		$options[CURLOPT_FRESH_CONNECT]=TRUE;
		$options[CURLOPT_SSL_VERIFYPEER]=FALSE;
		$options[CURLOPT_SSL_VERIFYHOST]=FALSE;

		$options[CURLOPT_CONNECTTIMEOUT]=5;
		$options[CURLOPT_TIMEOUT]=5;

		$this->options=$options;
		$this->setCookies($cookies);
	}

	public function open(){
		if( $this->curl != null ) throw new CurlLibException('Curl resource already opened');
		$this->curl = curl_init();
		curl_setopt_array($this->curl ,$this->options);
	}

        public function setCookies($tmp_file){
		if( !is_bool( $tmp_file ) || $tmp_file ){
			if( is_bool($tmp_file)){
				$tmp_file = tempnam( sys_get_temp_dir(), 'cookie_' );
			}
			$options[CURLOPT_COOKIEJAR] = $tmp_file;
			$options[CURLOPT_COOKIEFILE] = $tmp_file;
		}
		else{
			$options[CURLOPT_COOKIESESSION]=FALSE;
		}
		$this->setOptions($options);
        }

	public function setCurl(resource $curl){
		$this->curl = $curl;
	}

	public function getCurl(){
		return $this->curl;
	}

	public function setoptions($options_or_key,$value=NULL){
		if( is_array($options_or_key)){
			foreach( $options_or_key as $k => $v ){
				$this->options[$k] = $v;
			}
		}
		else{
			$this->options[$options_or_key] = $value;
			$options_or_key = array($options_or_key=>$value);
		}

		if( $this->curl != null ) curl_setopt_array($this->curl,$options_or_key);
	}

	protected $curl_info=NULL;
	public function execute_request($options=null){
		if( $this->curl == null ) throw new CurlLibException('Curl resource no present. You must first calling open() method');
		$ch = curl_copy_handle($this->curl);

		if( $options != NULL ){
			curl_setopt_array($ch ,$options);
		}
		$content = curl_exec($ch);

		$errno = curl_errno($ch);
		if($errno){
			throw new CurlException(curl_strerror($errno),$errno);
		}

		$this->curl_info = curl_getinfo($ch);

		curl_close($ch);

		return $content;
	}

	public function getLastTransferInfo(){
                return $this->curl_info;
        }

	public function getHttpCode(){
		return $this->curl_info['http_code'];
	}


	private function build_url($path,$params=NULL){
		if( empty ($path) ) {
			$url = $this->url;
		}
		else{
			$url = rtrim($this->url,'/').'/'.ltrim($path,'/');
		}
		if( $params != NULL ){
			$data = http_build_query($params);
			$url.="?$data";
		}
		return $url;
	}

	public function get($path,$params=null,$url_mode = TRUE){
		if( !$url_mode ){
			$options[CURLOPT_POSTFIELDS] = http_build_query($params);
			$params == NULL;
		}
		$options[CURLOPT_URL] = $this->build_url($path,$params);
		$response = $this->execute_request($options);
		return $response;
	}

	public function post($path,$params=null){
		$options[CURLOPT_URL] = $this->build_url($path);
		if( $params != NULL ){
			$data = http_build_query($params);
			$options[CURLOPT_POSTFIELDS]=$data;
		}
		$options[CURLOPT_POST]=TRUE;
		$response = $this->execute_request($options);
		return $response;
	}

	public function put($path,$params=null){
		$options[CURLOPT_URL] = $this->build_url($path);
		$options[CURLOPT_CUSTOMREQUEST]='PUT';
                if( $params != NULL ){
                        $data = http_build_query($params);
                        $options[CURLOPT_POSTFIELDS]=$data;
                }

		$response = $this->execute_request($options);
		return $response;
	}

	public function delete($path){
		$options[CURLOPT_URL] = $this->build_url($path);
		$options[CURLOPT_CUSTOMREQUEST]='DELETE';
		$response = $this->execute_request($options);
		return $response;
	}

	public function close(){
		if( $this->curl != null ){
			curl_close($this->curl);
		}
		$this->curl = NULL;
	}
}

class CurlException extends Exception {};
class CurlLibException extends Exception {};


function getParam($options,$name1,$name2,$default=null){
	if( isset( $options[$name1] ) ) return $options[$name1];
	if( isset( $options[$name2] ) ) return $options[$name2];
	return $default;
}

function exit_state($state,$msg=null,$usage=false){
	$STR_STATUS=$GLOBALS["STR_STATUS"];
	$INT_STATUS=$GLOBALS["INT_STATUS"];

	$ret=$state;
	if( is_string($state) ) $ret=$INT_STATUS[strtoupper($state)];
	else $state=$STR_STATUS[$ret];

	if( $msg != null ) echo "$state - $msg\n";
	if( $usage ) print_usage();
	exit($ret);
}

function calcul_state($a,$b=null){
	$STR_STATUS=$GLOBALS["STR_STATUS"];
	$INT_STATUS=$GLOBALS["INT_STATUS"];
	$PRIORITIES_STATES=$GLOBALS["PRIORITIES_STATES"];

	if(is_string($a) ) $a=$INT_STATUS[strtoupper($a)];
	if(is_string($b) ) $b=$INT_STATUS[strtoupper($b)];

	foreach($PRIORITIES_STATES as $value){
		$value = $INT_STATUS[$value];

		if( $value == $a || $value == $b ) return $value;
	}
	return $INT_STATUS["UNKNOWN"];
}

function print_help(){
	echo "This script is used for checking innovaphones health through web API\n\n";
	print_usage();
	echo "\n";
?>
Options :
	-H, --host=HOST             URL to access to Innovaphone Web API
	-A, --authfile=PATH         PATH for authentication file

	-u, --user=USER             User login to use. Replace authfile user login.
	-p, --pass=PASS             User password to use.  Replace authfile user password.

	-T, --type=TYPE             Test type
	-w, --warning=TRESHOLD      Warning treshold.
	-c, --critical=TRESHOLD     Critical treshold.

	-h, --help

Authfile :
	<USER>:PASS

Type :
	**To count call number**
	-T error_nb     -w MAXNB        -c MAXNB        Check calling nb into Alerting,Disconnecting
	-T call_nb      -w MAXNB        -c MAXNB        Check calling nb into Dialing,Calling,Connected

	**To check max time**
	-T error_time   -w MAXSECOND    -c MAXSECOND    Check max time for calling into Alerting,Disconnecting
	-T call_time    -w MAXSECOND    -c MAXSECOND    Check max time for calling into Dialing,Calling,Connected

	**To loss pkgs number**
	-T pkg_loss     -w MAXLOSS      -c MAXLOSS      Check avg PKG LOSS by minutes by call

	**To check metric (cpu or mem) usage**
	-T cpu          -w MAXPERCENT   -c MAXPERCENT   Check 5 minutes avg CPU usage
	-T mem          -w MAXPERCENT   -c MAXPERCENT   Check 5 minutes avg MEM usage

	**To check another metric usage**
	-T <integer>    -w MAXPERCENT   -c MAXPERCENT   Check 5 minutes avg <integer metric> usage
<?php
}

function print_usage(){
	$script_name = basename($_SERVER['PHP_SELF']);
	echo "USAGE : $script_name -H <HOST> -A <AUTHFILE> -T <TYPE> -w <WARNING> -c <CRITIC>\n";
	echo "      : $script_name -h for help\n";
}

function xml_attribute($object, $attribute)
{
    if(isset($object[$attribute]))
        return (string) $object[$attribute];
}

$STR_STATUS=array("OK","WARNING","CRITICAL","UNKNOWN");
$INT_STATUS=array_flip($STR_STATUS);
$PRIORITIES_STATES=array("CRITICAL","WARNING","UNKNOWN","OK");

//Récupération des parametres
$options = getopt('H:A:u:p:T:w:c:h',explode(',','host:,authfile:,type:,user:,pass:,warning:,critical:,help'));

$o_help 	= getParam($options,'h','help');

if($o_help !== NULL){
	print_help();
	exit($INT_STATUS['OK']);
}

$o_host 	= getParam($options,'H','host');
$o_file 	= getParam($options,'A','authfile');
$o_passname = getParam($options,'P','passname',$o_host);

if( $o_host == NULL ) exit_state('UNKNOWN','Option -H, --host is missing',TRUE);

$o_user = $o_pass = NULL;
if( $o_file != NULL ){
	$found = FALSE;
	$content = @file_get_contents($o_file) or exit_state('UNKNOWN',"Unable to access to authfile '$o_file'");
	$content = explode("\n",$content);
	foreach($content as $row){
		if( preg_match('/^\s*#/',$row)) continue;  # Commentaire
		$fields = explode(':',$row,'2');
		$o_user = $fields[0];
		$o_pass = $fields[1];
		$found = TRUE;
		break;
	}
	if ( !$found ) exit_state('UNKNOWN',"No entry found into file $o_file");
}
$o_user 	= getParam($options,'u','user',$o_user);
$o_pass 	= getParam($options,'p','pass',$o_pass);

if( $o_user == NULL ) exit_state('UNKNOWN','Missing user login, must be set into authfile -A, --authfile or with -u, --user option',TRUE);
if( $o_pass == NULL ) exit_state('UNKNOWN','Missing user password, must be set into authfile -A, --authfile or with -p, --pass option',TRUE);

$o_type		= getParam($options,'T','type');
if( $o_type == NULL ) exit_state('UNKNOWN','Option -T, --type is missing',TRUE);

$o_warn 	= getParam($options,'w','warning');
$o_crit 	= getParam($options,'c','critical');

if(!isset($o_warn )||!isset($o_crit)){
	exit_state('UNKNOWN',"Tresholds options -w or -c are missing");
}
if( $o_warn > $o_crit){
	exit_state('UNKNOWN',"Treshold options -w $o_warn can't be bigger than -c $o_crit");
}

$type = strtolower($o_type);
if( in_array($type,explode(',','error_nb,call_nb,error_time,call_time,pkg_loss'))){
	$request_type = 'currents_calls';
}
elseif(in_array($type,explode(',','cpu,mem'))){
	$request_type = 'metrics';
	if( $type == 'cpu') 	$metric_id = 0;
	elseif( $type == 'mem') $metric_id = 2;
}
elseif(is_numeric($o_type)){
	$request_type = 'metrics';
	$metric_id = $o_type;
}
else{
	exit_state('UNKNOWN',"Unknown -T, --type '$o_type'",TRUE);
}

//Récupération des données
$httpClient = new HttpClient($o_host);
$httpClient->setoptions(CURLOPT_USERPWD,"$o_user:$o_pass");
$httpClient->open();

$data = array();
try{
	if( $request_type == 'currents_calls' ){
 //    $response = $httpClient->get('/RELAY0/mod_cmd.xml?cmd=xml-calls&xsl=relay_calls_view.xsl&help=Gateway/Calls');
        $response = $httpClient->get('/PBX0/ADMIN/mod_cmd_login.xml?cmd=show&call=fm&xsl=pbx_calls_view.xsl&help=PBX/Calls');

		if( preg_match('/<HTML><HEAD><TITLE>([45]\d\d [^<]+)/',$response,$matches)){
			$str = $matches[1];
			$response = explode("\n",$response);
			array_pop($response); //</BODY></HTML>
			$str.= ' - '.array_pop($response); 
			exit_state('UNKNOWN',"Http response error : $str");
		}

		//$response = file_get_contents('call1.xml');
		$xml = simplexml_load_string($response);
		$data['info'] = (array) $xml->attributes();
                if( !isset($data['info']['@attributes'])) exit_state('UNKNOWN','Unable to parse XML response');

		$data['time']=xml_attribute($xml,'time');

		$data['calls'] = array();
		foreach($xml->children() as $xml_call){
                        $data_row['state']      = xml_attribute($xml_call,'state');
			$data_row['conf']	= xml_attribute($xml_call,'conf');
			if( !isset($data_row['state'] ) || $data_row['state'] == 'Complete') continue;
			$data_row['time'] 	= $data['time'] - xml_attribute($xml_call->children()[0],'time');
			$data_row['codec'] = array();
			foreach( $xml_call->children() as $traffic){
				$rtp = (array) $traffic->media->rtp;
				if( isset($rtp['@attributes']['loss']) ){
					$data_row['codec'][] = $rtp['@attributes']['loss'];
				}
			}
			if( !isset( $data['calls'][$data_row['conf']]) ){
				$data['calls'][$data_row['conf']] = $data_row;
			}
			else{
				$status_priority = array('Connected'=>0,'Calling'=>1,'Dialing'=>2,'Disconnecting'=>3,'Alerting'=>4);

				if( $data['calls'][$data_row['conf']]['time'] < $data_row['time'] ){
					$data['calls'][$data_row['conf']]['time'] = $data_row['time'];
				}
				if( $status_priority[$data['calls'][$data_row['conf']]['state']] < $status_priority[$data_row['state']]){
					$data_row['state'] = $status_priority[$data_row['state']];
				}
				$data['calls'][$data_row['conf']]['codec'] = array_merge($data['calls'][$data_row['conf']]['codec'],$data_row['codec']);
			}
		}
	}
	else if( $request_type == 'metrics' ){
		$response = $httpClient->get('/LOG0/CNT/mod_cmd.xml?cmd=xml-count&pos=23&x='.$metric_id);
                if( preg_match('/<HTML><HEAD><TITLE>([45]\d\d [^<]+)/',$response,$matches)){
                        $str = $matches[1];
                        $response = explode("\n",$response);
                        array_pop($response); //</BODY></HTML>
                        $str.= ' - '.array_pop($response);
                        exit_state('UNKNOWN',"Http response error : $str");
                }

	//	$response = file_get_contents('metrics2.xml');
		$xml = simplexml_load_string($response);
		$data['info'] = (array) $xml->attributes();
	        if( !isset($data['info']['@attributes'])) exit_state('UNKNOWN','Unable to parse XML response');
		$data['info'] = $data['info']['@attributes'];
		$hours = 0;
		foreach( $xml->children() as $xml_hours){
			$min=0;
			foreach( $xml_hours->children() as $xml_min){
				$data[$hours][$min] =  xml_attribute($xml_min,'c');
				$min++;
			}
			$hours++;
		}
	}
	$httpClient->close();
}
catch(Exception $ex){
	exit_state('UNKNOWN',"Unable to get data from Innovaphone device '$o_host'. ".$ex->getMessage());
}

if( in_array($type, explode(',','error_nb,call_nb'))){
	$search_states['error_nb']	= array('Alerting','Disconnecting');
	$search_states['call_nb']	= array('Dialing','Calling','Connected');

	$nb_call = array_fill_keys(array('TOTAL','FOUND'),0);
	$states_nb = array_fill_keys(array('Dialing','Calling','Connected','Disconnecting','Alerting'),0);
    $ret_unknown = FALSE;
	foreach($data['calls'] as $call){
        if( empty( $call['state'] )) continue;
		$nb_call['TOTAL']++;
        if( !isset($states_nb[$call['state']])){
            echo "UNKNOWN state : ".$call['state']."\n";
            $ret_unknown = TRUE;
        }
		$states_nb[$call['state']]++;
		if( in_array($call['state'],$search_states[$type]) ) $nb_call['FOUND']++;
	}

	if( $nb_call['FOUND'] >= $o_crit ){
			$state = $INT_STATUS['CRITICAL'];
			$treshold_str =  ">= $o_crit";
	}
	else if( $nb_call['FOUND'] >= $o_warn ){
		$state = $INT_STATUS['WARNING'];
		$treshold_str =  ">= $o_warn";
	}
	else{
		$state = $INT_STATUS['OK'];
		$treshold_str =  "< $o_warn";
	}

    if( $ret_unknown && $state == $INT_STATUS['OK']) $state =  $INT_STATUS['UNKNOWN'];

	$str = $nb_call['FOUND'] . ' calls found'." ($treshold_str)". ' with states ('.implode(',',$search_states[$type]).'). ';

	$str.= 'Total '.$nb_call['TOTAL'];
	$str_states = array();
	foreach($states_nb as $key => $value){
		$str_states[] = "$key=$value";
	}
	$str.= ' ('.implode(',',$str_states).').';
	$str.= "|w:$o_warn,c:$o_crit";
	exit_state($state,$str);
}
else if( in_array($type, explode(',','error_time,call_time'))){
	$search_states['error_time']	= array('Alerting','Disconnecting');
	$search_states['call_time']	= array('Dialing','Calling','Connected');
    $search_states['all'] = array_merge($search_states['error_time'],$search_states['call_time']);

	$warn_call = array();
	$total_state_nb=0;
	$biggest_time=0;
	$initial_time=$data['info']['@attributes']['time'];
	$state_call = array(0,0,0); //OK,WARN,CRIT
    $ret_unknown = FALSE;
	foreach($data['calls'] as $call){
	if( empty( $call['state'] )) continue;
        if( !in_array($call['state'],$search_states['all']) ){
            echo "UNKNOWN state : ".$call['state']."\n";
            $ret_unknown = TRUE;
        }
		if( !in_array($call['state'],$search_states[$type])) continue;
		$total_state_nb++;
		if( $call['time'] > $biggest_time ) $biggest_time = $call['time'];

		if( $call['time'] >= $o_crit ) 		$state_call[$INT_STATUS['CRITICAL']]++;
		else if( $call['time'] >= $o_warn ) $state_call[$INT_STATUS['WARNING']]++;
		else $state_call[$INT_STATUS['OK']]++;
	}

	$total_count = count($data['calls']);
	$total_treshold = 0;

	if( $state_call[$INT_STATUS['CRITICAL']] > 0 ){
		$state = 'CRITICAL';
		$treshold_str =  ">= $o_crit";
		$total_treshold = $state_call[$INT_STATUS['CRITICAL']];
	}
	else if( $state_call[$INT_STATUS['WARNING']]  > 0 ){
		$treshold_str =  ">= $o_warn";
		$total_treshold = $state_call[$INT_STATUS['WARNING']];
		$state = 'WARNING';
	}
	else{
		$treshold_str =  "< $o_warn";
		$state = 'OK';
	}

    if( $ret_unknown && $state == $INT_STATUS['OK']) $state =  $INT_STATUS['UNKNOWN'];

	$str = "$total_treshold call (with time $treshold_str seconds) / $total_state_nb with states (".implode(',',$search_states[$type]).').';
	$str.= " Biggest time is $biggest_time seconds";
	$str.= sprintf("|ok:%d,warn:%d,cri:%d,tw:%d,tc:%d" ,$state_call[$INT_STATUS['OK']],$state_call[$INT_STATUS['WARNING']],$state_call[$INT_STATUS['CRITICAL']],$o_warn,$o_crit);
	exit_state($state,$str);
}
else if( $type == 'pkg_loss' ){
	$total_count = 0;
	$total_loss = 0;
	$total_time = 0;
	$max_loss = 0;

	$str_call = array();
	$str_calling_states = array();
	$all_states = $INT_STATUS['OK'];
	foreach($data['calls'] as $call){
		$time = $call['time'];
		$state = $call['state'];
		$loss = 0;
		$dir = 0;
		foreach($call['codec'] as $key => $value){
			$loss += $value;
		}

		$str_calling_states[$state]=$state;
		$total_count++;
		$total_loss += $loss;
		$total_time += $time;

		$by_min = round($loss/($time/60),2);

		if( $by_min > $max_loss ) $max_loss = $by_min;

		if( $by_min > $o_crit){
			$str_call[] = "CRITICAL - Calling state $state loss $loss packets/$time seconds ($by_min packets/min > $o_crit)";
			$current_state = $INT_STATUS['CRITICAL'];
		}
		else if( $by_min > $o_warn){
			$str_call[] = "WARNING - Calling state $state loss $loss packets/$time seconds ($by_min packets/min > $o_warn)";
			$current_state = $INT_STATUS['WARNING'];
		}
		else{
			$str_call[] = "OK - Calling state $state loss $loss packets/$time seconds ($by_min packets/min <= $o_warn)";
			$current_state = $INT_STATUS['OK'];
		}
		$all_states = calcul_state($current_state,$all_states);
	}
	$str_calling_states = implode(',',$str_calling_states);
	$str_call = implode("\n",$str_call);

	$total_min = $total_time/60;
	$total_by_min = round($total_loss/$total_min,2);
	$total_min = round($total_min,2);

	$str = "Current $total_count total ($str_calling_states) call loss $total_loss packets/$total_min minutes ($total_by_min packets/min)\n";
	$str.= $str_call;
	$str.= "\n|max:$max_loss;w:$o_warn;c:$o_crit";
	exit_state($all_states,$str);
}
else if( $request_type == 'metrics' ){
	$nb_get = 5;
	$last_data = array_pop($data);
	$last_data = array_merge(array_pop($data),$last_data);
	$key = count($last_data)-1;
	$nb = 0;
	$ret = array();
	do{
		if( $last_data[$key] != NULL ){
			array_unshift($ret, $last_data[$key]);
			$nb++;
		}
		$key--;
	}while($nb<$nb_get && $key > 0);
	$percent_max = max($ret);
	$percent_min = min($ret);
	$percent_avg = round(array_sum($ret)/$nb_get,2);

	$name = $data['info']['n'];
	$total = $data['info']['s'];
	$total_str = $data['info']['t'];
	$unit = str_replace($total,'',$data['info']['t']);

	$str = "$name : (Total : $total_str) ${percent_avg}% used";
	$stats = "min:$percent_min;avg:$percent_avg;max:$percent_max;warn:$o_warn;crit:$o_crit";
	if( $percent_avg >= $o_crit ){
		$treshold_str = ">= $o_crit%";
		$state = $INT_STATUS['CRITICAL'];
	}
	else if( $percent_avg >= $o_warn ){
		$treshold_str = ">= $o_warn%";
		$state = $INT_STATUS['WARNING'];
	}
	else{
		$treshold_str = "< $o_warn%";
		$state = $INT_STATUS['OK'];
	}
	exit_state($state,"$str $treshold_str |$stats");
}
