#!/usr/bin/perl -w
############################## check_snmp_hp5400_temp.pl ########
my $Version='1.0';
# Date : 19 février 2019
# Author  : Nicolas Biersart
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;
use ESIA::CacheFile;
use JSON;
my $PROGNAME='check_snmp_aruba_ap.pl';
my $PROGDESC = "Check Aruba Access Point CPU, ram and environnement sensors";
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;


my $return=0;
my ($opt_h,$opt_V,$debug,$json);
my $warning='';
my $critical='';
my $type = 'env';
my $path='/tmp/aparuba';
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";
my $cntrl_link;
ESIA::PLUGIN::arm_end_signal($TIMEOUT);

#IP
my $oid_aruba_ip     = "1.3.6.1.4.1.14823.2.3.3.1.2.1.1.3";
my $oid_aruba_serial = "1.3.6.1.4.1.14823.2.3.3.1.2.1.1.4";
my $oid_aruba_model  = "1.3.6.1.4.1.14823.2.3.3.1.2.1.1.6";
my $oid_aruba_name   = "1.3.6.1.4.1.14823.2.3.3.1.2.1.1.2";
my $oid_aruba_status = "1.3.6.1.4.1.14823.2.3.3.1.2.1.1.11";
#CPU
my $oid_ap_cpu = "1.3.6.1.4.1.14823.2.3.3.1.2.1.1.7"; # Procurve CPU Counter
# MEM
my $oid_ap_mem_total	    = "1.3.6.1.4.1.14823.2.3.3.1.2.1.1.10"; # Total Bytes
my $oid_ap_mem_free		= "1.3.6.1.4.1.14823.2.3.3.1.2.1.1.8"; # Free Bytes

#controlleur
my $oid_cntrl_name ='1.3.6.1.4.1.14823.2.3.3.1.1.2.0';
my $oid_cntrl_version ='1.3.6.1.4.1.14823.2.3.3.1.1.4.0';
my $oid_cntrl_ip ='1.3.6.1.4.1.14823.2.3.3.1.1.5.0';

#client
my $oid_client_ip ='1.3.6.1.4.1.14823.2.3.3.1.2.4.1.3';
sub Trace
{
	 my($debug,$message) = @_;
	 if($debug)
	 {
	 	print "DEBUG: $message\n";
	 }
}
#
#MAIN
#
&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"D"   => \$debug,"debug"     => \$debug,
	"s"   => \$cntrl_link,"standalone" => \$cntrl_link,
    "T=s" => \$type,"type=s" => \$type,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	"d=s" => \$path,"directory=s"=>\$path,
	"J" => \$json,	 "json" => \$json,
);
# -h & --help print help
if ($opt_h || !defined($snmp))
{
	#affiche l'aide par défaut
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-T state');
	my @arr;
	push(@arr,["-T","--type","type of test [cpu,mem,env,client,inventory] (default: cpu)" ]);
	push(@arr,["-w","--warning","warning threshold in percent" ]);
	push(@arr,["-c","--critical","critical threshold in percent"]);
	push(@arr,["-d","--directory","directory for tmp file (default: $path)" ]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
#on interroge en SNMP pour récupérer les informations
Trace($debug,"GET SNMP REQUEST");
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if($Sysoid_value==-1)
{
	print ("UNKNOWN: Host not SNMP response $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
# test des paramètre par défaut
if($warning eq '' )
{
    if($type eq 'cpu' || $type eq 'mem')
    {
        $warning=80;
    }
}
if($critical eq '')
{
    if($type eq 'cpu'|| $type eq 'mem')
    {
        $critical=80;
    }
}
#
#HP procurve switch
#
if($Sysoid_value == 14823)
{

	my $hostname = $snmp->hostname();
	my $filecache = $hostname."_information.json";
	Trace($debug,"LOAD CACHE FILE $path/ $filecache ");
	my $cache = ESIA::CacheFile->new($path,$filecache,15);
	my $apInfo;
	if($cache->isValid() == 1)
	{
		my $data = $cache->getData();
		$apInfo = decode_json($data);
		Trace($debug,"USE CACHE FILE");
	}
	else
	{
		Trace($debug,"RELOAD CACHE FILE");
		my $res = $snmp->get_table(Baseoid =>$oid_aruba_ip);
		my $index='-1';
    	foreach my $key ( keys %$res)
		{
		    if($key =~ /$oid_aruba_ip\.([0-9\.]*)$/)
			{
				my $ip = $$res{"$key"};
				my $ind = $1;
				Trace($debug,"INDEX $ind");
				if($ip eq $hostname)
				{
					$index = $ind;
					Trace($debug,"INDEX FOUND $ind");
				}
			}
		}
		if($index ne -1)
		{
			my %hash;
			$hash{'index'}=$index;
			$hash{'serial'} = ESIA::SNMP::get($snmp,"$oid_aruba_serial.$index");
			$hash{'model'} = ESIA::SNMP::get($snmp,"$oid_aruba_model.$index");
			$hash{'name'} = ESIA::SNMP::get($snmp,"$oid_aruba_name.$index");
			$apInfo= \%hash;
			my $json = encode_json \%hash;
			$cache->writeData($json);
		}
		else
		{
			print ("UNKNOWN: Management IP does not correspond\n");
			exit $ERRORS{'UNKNOWN'};
		}
	}
    if($type eq 'env')
    {
        my $state = 'OK';
        my $status = ESIA::SNMP::get($snmp,"$oid_aruba_status.".$apInfo->{'index'});
        my $message ='up';
        if($status != 1 )
        {
        	$message ='down';
        	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
        	$state='CRITICAL';
        }
		print "$state: ".$apInfo->{'name'}.' model '.$apInfo->{'model'}.' (SN:'.$apInfo->{'serial'}.") is $message\n";

		my $cntrl_ip = ESIA::SNMP::get($snmp,"$oid_cntrl_ip");
		if($cntrl_ip eq '0.0.0.0')
		{
			if(!defined($cntrl_link))
			{
				print "OK: AP is standalone mode\n";
				$return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
			}
			else
			{
				print "CRITICAL: AP is standalone mode\n";
				$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
			}
		}
		else
		{
			my $cntrl_name = ESIA::SNMP::get($snmp,"$oid_cntrl_name");
			my $cntrl_ver = ESIA::SNMP::get($snmp,"$oid_cntrl_version");
			print "OK: AP is linked to $cntrl_name (version: $cntrl_ver). Linked IP $cntrl_ip\n";
		}
    }
    elsif($type eq 'cpu')
    {
        my $state = 'OK';
        my $threshold = ESIA::SNMP::get($snmp,"$oid_ap_cpu.".$apInfo->{'index'});
        my $message="(< $warning)";
        if($threshold >= $critical)
        {
            $state='CRITICAL';
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
            $message="(> $critical)";
        }
        elsif($threshold >= $warning)
        {
            $state='WARNING';
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
            $message="(> $warning)";
        }
        print "$state : CPU used $threshold% $message|cpu=$threshold";
    }
    elsif($type eq 'mem')
    {
        my $state = 'OK';
        my $mem_total = ESIA::SNMP::get($snmp,"$oid_ap_mem_total.".$apInfo->{'index'});
        my $mem_free = ESIA::SNMP::get($snmp,"$oid_ap_mem_free.".$apInfo->{'index'});
		my $threshold = 100 - (($mem_free / $mem_total)*100);
		$threshold = int($threshold);
        my $message="(< $warning)";
        if($threshold >= $critical)
        {
            $state='CRITICAL';
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
            $message="(> $critical)";
        }
        elsif($threshold >= $warning)
        {
            $state='WARNING';
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
            $message="(> $warning)";
        }
        print "$state : Memory used $threshold% $message|mem=$threshold";
    }
    elsif($type eq 'client')
    {
        my $state = 'OK';
		my $res = $snmp->get_table(Baseoid =>$oid_client_ip);
		my $nbr = 0;
    	foreach my $key ( keys %$res)
		{
			$nbr++;
		}

        my $message="(< $warning)";
        if($nbr >= $critical)
        {
            $state='CRITICAL';
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
            $message="(> $critical)";
        }
        elsif($nbr >= $warning)
        {
            $state='WARNING';
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
            $message="(> $warning)";
        }
        print "$state : There are $nbr client connected $message|totalclient=$nbr";
    }
    elsif($type eq 'inventory')
    {
        my @output;
		my %hash = (
			'name' => 'Access point '.$apInfo->{'model'},
			'description'=>'',
			'manufacturer'=>'Aruba',
			'model' => $apInfo->{'model'},
			'type'=>'CHASSIS',
			'serial' => $apInfo->{'serial'},
			'hardwareVersion' => '',
			'firmwareVersion' => '',
		);
		push(@output, \%hash);
		if($json)
		{
			my $out = encode_json(\@output);
			print $out;
		}
		else
		{
		    my $size = @output;
		    if($size != 0)
		    {
		        foreach my $line ( @output)
		        {

		            my $hash = \%$line;
		            print "name: ".$hash->{name}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
		            print ", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
		        }
		        exit 0;
		    }
		}
    }
}
else
{
    print "OID is unknown. Model not support contact your support\@esia-sa.com\n";
    $snmp->close;
    exit $ERRORS{"UNKNOWN"};
}
exit($return);
