#!/usr/bin/perl
use strict;
use warnings;
use lib "/usr/local/esia/plugins/lib/perl"  ;
#use lib "/root/libperl/"  ;
use vars qw($PROGNAME);
use Net::SNMP;
use Math::Round;
use Getopt::Long;
use ESIA::SNMP;
use ESIA::PLUGIN;
use ESIA::CacheFile;
use ESIA::INVENTORY;
use JSON;
#nom du programme & description
my $PROGNAME = "check_snmp_cisco_switch.pl";
my $PROGDESC = "Cisco switches by SNMP";
my $PROGVERSION = "1.0";
#Chargement du timer par défaut et des erreurs
# a installer apt install libmath-round-perl
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

ESIA::PLUGIN::arm_end_signal($TIMEOUT);

Getopt::Long::Configure( "bundling" );
my ($opt_h,$opt_V,$json);
my $community = "public";
my $snmp_version = 1;
my ($hostname);
my $warning=80;
my $critical=90;
my $return =0;
my $type;
my $showIgnored;

my $oid_sysObjectOID                = "1.3.6.1.2.1.1.2.0";

# CISCO-ENVMON-MIB
my $oid_ciscoFanDesc 		= "1.3.6.1.4.1.9.9.13.1.4.1.2"; 		#Description
my $oid_ciscoFanStatut 		= "1.3.6.1.4.1.9.9.13.1.4.1.3"; 		#Status normal(1), warning(2), critical(3), shutdown(4), notPresent(5), notFunctioning(6)
my $oid_ciscoPowerDesc 		= "1.3.6.1.4.1.9.9.13.1.5.1.2"; 		#Description
my $oid_ciscoPowerStatut    = "1.3.6.1.4.1.9.9.13.1.5.1.3"; 		#Status normal(1), warning(2), critical(3), shutdown(4), notPresent(5), notFunctioning(6)
my $oid_ciscoTempDesc 		= "1.3.6.1.4.1.9.9.13.1.3.1.2"; 		#Description
my $oid_ciscoTempStatut     = "1.3.6.1.4.1.9.9.13.1.3.1.6";         #Status normal(1), warning(2), critical(3), shutdown(4), notPresent(5), notFunctioning(6)
my $oid_ciscoTempValue      = "1.3.6.1.4.1.9.9.13.1.3.1.3";         #Value


# Cisco CISCO-ENTITY-SENSOR-MIB
my $oid_cisco_ios_xe_physicaldescr      = "1.3.6.1.2.1.47.1.1.1.1.7";
my $oid_cisco_ios_xe_type     	    	= "1.3.6.1.4.1.9.9.91.1.1.1.1.1";
my $oid_cisco_ios_xe_precision	  	    = "1.3.6.1.4.1.9.9.91.1.1.1.1.3";
my $oid_cisco_ios_xe_value     	    	= "1.3.6.1.4.1.9.9.91.1.1.1.1.4";
my $oid_cisco_ios_xe_status     	 	= "1.3.6.1.4.1.9.9.91.1.1.1.1.5";   #Status normal(1), unavailable(2), nonoperational(3)
my $oid_cisco_ios_xe_ThresholdEval     	= "1.3.6.1.4.1.9.9.91.1.2.1.1.5";   # 1 evaluation true, 2 faux

# Cisco OLD-CISCO-CHASSIS-MIB
my $oid_cisco_chassis_card_descr		= "1.3.6.1.4.1.9.3.6.11.1.3"; 		# Chassis card description
my $oid_cisco_chassis_card_slot		    = "1.3.6.1.4.1.9.3.6.11.1.7"; 		# Chassis card slot number
my $oid_cisco_chassis_card_state		= "1.3.6.1.4.1.9.3.6.11.1.9";       # operating status of card - 1 : Not specified, 2 : Up, 3: Down, 4 : standby

## FAN
my $oid_ciscoEnvMonFanStatusDesc   = "1.3.6.1.4.1.9.6.1.101.83.1.1.1.2";
my $oid_ciscoEnvMonFanState        = "1.3.6.1.4.1.9.6.1.101.83.1.1.1.3";

## PSU
my $oid_ciscoEnvMonSupplyStatusDescr    = "1.3.6.1.4.1.9.6.1.101.83.1.2.1.2";
my $oid_ciscoEnvMonSupplyState          = "1.3.6.1.4.1.9.6.1.101.83.1.2.1.3";

## TEMP
my $oid_rlEnvFanDataTemp                = "1.3.6.1.4.1.9.6.1.101.83.5.1.1.2";

## LOAD
my $oid_cisco_cpu_5m = "1.3.6.1.4.1.9.2.1.58.0"; # Cisco CPU load (5min %)
my $oid_cisco_cpu_1m = "1.3.6.1.4.1.9.2.1.57.0"; # Cisco CPU load (1min %)
#my $oid_cisco_cpu_5s = "1.3.6.1.4.1.9.2.1.56.0"; # Cisco CPU load (5sec %)

my $oid_cisco_sb_cpu_5m = "1.3.6.1.4.1.9.6.1.101.1.9.0"; # Cisco Small Business CPU load (5min %)
my $oid_cisco_sb_cpu_1m = "1.3.6.1.4.1.9.6.1.101.1.8.0"; # Cisco Small Business CPU load (1min %)
#my $oid_cisco_sb_cpu_5s = "1.3.6.1.4.1.9.6.1.101.1.7.0"; # Cisco Small Business CPU load (5sec %)

## MEM
my $cisco_mem_pool      = "1.3.6.1.4.1.9.9.48.1.1.1"; # Cisco memory pool
my $cisco_index         = "1.3.6.1.4.1.9.9.48.1.1.1.2"; # memory pool name and index
my $cisco_valid         = "1.3.6.1.4.1.9.9.48.1.1.1.4"; # Valid memory if 1
my $cisco_used          = "1.3.6.1.4.1.9.9.48.1.1.1.5"; # Used memory
my $cisco_free          = "1.3.6.1.4.1.9.9.48.1.1.1.6"; # Free memory
#FLASH

my $oid_cisco_sb_FileTotalSizeOfFlash = '1.3.6.1.4.1.9.6.1.101.96.4.0'; # in bytes.
my $oid_cisco_sb_FileFreeSizeOfFlash = '1.3.6.1.4.1.9.6.1.101.96.5.0';  # in bytes.
my $oid_cisco_flash_name_partition = "1.3.6.1.4.1.9.9.10.1.1.4.1.1.10";
my $oid_cisco_flash_size = "1.3.6.1.4.1.9.9.10.1.1.4.1.1.4"; # in bytes.
my $oid_cisco_flash_free = "1.3.6.1.4.1.9.9.10.1.1.4.1.1.5"; # in bytes.
my $oid_cisco_flash_extended_size = "1.3.6.1.4.1.9.9.10.1.1.4.1.1.13"; # in bytes.
my $oid_cisco_flash_extended_free = "1.3.6.1.4.1.9.9.10.1.1.4.1.1.14"; # in bytes.

my %ciscoEnvMonState = (
    "1" => "normal(1)",
    "2" => "warning(2)",
    "3" => "critical(3)",
    "4" => "shutdown(4)",
    "5" => "notPresent(5)",
    "6" => "notFunctioning(6)",
);

my %ciscoEnvMonStateToEsia = (
    "1" => "OK",
    "2" => "WARNING",
    "3" => "CRITICAL",
    "4" => "CRITICAL",
    "5" => "IGNORE",
    "6" => "CRITICAL",
);
my %envmon_status_message  = ( "1"  => "normal","2" => "warning","3"  => "critical","4"  => "shutdown","5"  => "notPresent","6"  => "notFunctioning");
my %envmon_status = ( "1"  => "OK","2" => "WARNING","3"  => "CRITICAL","4"  => "UNKNOWN","5"  => "OK","6"  => "CRITICAL");

my @cisco_ios_xe_type_text = ("not_specified","other","unknown","voltsAC","voltsDC","amperes","watts","hertz","celsius","percent","rpm","cmm","truthvalue","specialEnum","dBm"); # correspondance de la valeur type

my %chassis_status_message  = ( "1"  => "Not specified","2" => "Up","3"  => "Down","4"  => "standby");
my %chassis_status = ( "1"  => "UNKNOW","2" => "OK","3"  => "CRITICAL","4"  => "OK");

my $oid_cisco_asr_sensors    ='1.3.6.1.4.1.9.9.91.1.1.1.1.1';
my $oid_cisco_asr_sensors_val='1.3.6.1.4.1.9.9.91.1.1.1.1.4';
my $oid_cisco_asr_sensors_prec='1.3.6.1.4.1.9.9.91.1.1.1.1.3';
my $oid_cisco_asr_sensors_multi      ='1.3.6.1.4.1.9.9.91.1.1.1.1.2';
my $oid_cisco_asr_sensors_link_entity='1.3.6.1.4.1.9.9.91.1.1.1.1.8';
my $oid_cisco_asr_sensors_threshold='1.3.6.1.4.1.9.9.91.1.2.1.1.5';
my $oid_cisco_asr_sensors_status   ='1.3.6.1.4.1.9.9.91.1.1.1.1.5';
my $oid_cisco_asr_version_os="1.3.6.1.4.1.9.9.92.1.1.1.9";
my $oid_entity_name='1.3.6.1.2.1.47.1.1.1.1.2';
my %ciscoAsrDataType =(
    "1" => "other",
    "2" => "unknown",
    "3" => "voltsAC",
    "4" => "voltsDC",
    "5" => "amperes",
    "6" => "watts",
    "7" => "hertz",
    "8" => "celsius",
	"9" => "percentRH",
	"10" => "rpm",
	"11" => "cmm",
	"12" => "truthvalue",
	"13" => "specialEnum",
	"14" => "dBm",
	"15" => "dB",
);

# CISCO-PROCESS-MIB

# Cisco ASRx cpu/load & Cisco Nexus
my $cisco_asr_cpu_total_index 		   = ".1.3.6.1.4.1.9.9.109.1.1.1.1.1"; #cpmCPUTotalIndex
my $cisco_asr_cpu_total_physical_index = ".1.3.6.1.4.1.9.9.109.1.1.1.1.2"; #cpmCPUTotalPhysicalIndex
my $cisco_asr_cpu_1m 				   = ".1.3.6.1.4.1.9.9.109.1.1.1.1.7"; #cpmCPUTotal1minRev
my $cisco_asr_cpu_5m 				   = ".1.3.6.1.4.1.9.9.109.1.1.1.1.8"; # cpmCPUTotal5minRev

sub getCiscoProcessCPUUsage
{ 
    my($snmp,$sysOid_value,$warning,$critical,$match ) = @_;
    my $is_asr = 0;
    my $is_nexus = 0;

    if( $sysOid_value =~ /1\.3\.6\.1\.4\.1\.9\.1\.([0-9]*)/ )
    {
        if($1 == 1017 ||  $1 == 1018 ||  $1 == 1036 ||  $1 == 1037)
        {
        	 $is_asr = 1;
        } 
    }
    elsif(   $sysOid_value =~ /1\.3\.6\.1\.4\.1\.9\.12\.3\.1\.3/)
    {
        $is_nexus = 1;
    }

        if($warning > $critical )
        {
            print "UNKNOWN : You must specify correct warning,critical tresholds.\n";
            exit $ERRORS{UNKNOWN};
        }
        my $load_time;
        my $load_value;

        if( !defined($match) || $match eq '5' || $match eq '5m' || $match eq '300s' )
        {
            $load_time = 5;
            if($is_asr == 1 || $is_nexus == 1)
            {
            	my $resultat = $snmp->get_table(Baseoid => $cisco_asr_cpu_5m);
				if (!defined($resultat))
				{
			   		printf("ERROR: Description table : %s.\n", $snmp->error);
			   		$snmp->close;
			   		exit $ERRORS{"UNKNOWN"};
				}
				else
				{
					my $i = 0;
					foreach my $key ( keys %$resultat)
					{
						my $coreload = $$resultat{$key};
						if(defined($coreload))
						{
							$load_value += $coreload;
							$i++;
						}
					}
					$load_value = sprintf("%.1f",$load_value/$i);
				}
            }
            else
            {
            	$load_value = ESIA::SNMP::get($snmp,"$oid_cisco_cpu_5m");
            }

        }
        elsif( $match eq '1' || $match eq '1m' || $match eq '60s' )
        {
            $load_time = 1;
            if($is_asr == 1)
            {
            	my $resultat = $snmp->get_table(Baseoid => $cisco_asr_cpu_1m);
				if (!defined($resultat))
				{
			   		printf("ERROR: Description table : %s.\n", $snmp->error);
			   		$snmp->close;
			   		#exit $ERRORS{"UNKNOWN"};
			   		return -1;
				}
				else
				{
					my $i = 0;
					foreach my $key ( keys %$resultat)
					{
						my $coreload = $$resultat{$key};
						if(defined($coreload))
						{
							$load_value += $coreload;
							$i++;
						}
					}
					$load_value = sprintf("%.1f",$load_value/$i);
				}
            }
            else
            {
            	$load_value = ESIA::SNMP::get($snmp,"$oid_cisco_cpu_1m");
            }
        }
        else
        {
            print "UNKNOWN : You must specify correct -m, --match for load ( 1,5 | 1m,5m | 60s,300s )\n";
            exit $ERRORS{UNKNOWN};
        }

        my $out = '';
        my $output = '';
        my $state = 'UNKNOWN';
        if( $load_value < $warning )
        {
            $state = 'OK';
            $output = "$state: CPU used $load_value% (load average $load_time minutes) (< $warning%)";
        }
        elsif( $load_value < $critical )
        {
            $state = 'WARNING';
            $output = "$state: CPU used $load_value% (load average $load_time minutes) (>= $warning%)";
        }
        else
        {
            $state = 'CRITICAL';
            $output = "$state: CPU used $load_value% (load average $load_time minutes) (>= $critical%)";
        }
        $load_value = sprintf("%d",$load_value); 
		$output.="|cpu=$load_value 5m;warning=$warning;critical=$critical;\n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});

        my $found = 1;
        return $found,$return,$output;
}

# CISCO-ENHANCED-MEMPOOL
my $cempMemPoolUsed = '1.3.6.1.4.1.9.9.221.1.1.1.1.7';
my $cempMemPoolFree = '1.3.6.1.4.1.9.9.221.1.1.1.1.8';

sub getCiscoEnhancedMemPool
{
    my($snmp,$warning,$critical,$match) = @_;
	my $ret = 0;
	my $out = "";
	my $found = 0;

    my $output = '';
    my $graphdata = '';

	my $result = $snmp->get_table(Baseoid => $cempMemPoolUsed);
    if (defined($result))
    {
    	$found = 1;
        my $total_percent = 0;
        my $nb = 0;
		foreach my $key ( sort keys %$result)
		{
			#on récupère l'index
			if($key=~ /([0-9]*)\.([0-9]*)$/ )
			{
                my $index =$2;
                my $hw = $1;
                my $used = $$result{"$key"};
                my $free = ESIA::SNMP::get($snmp,"$cempMemPoolFree.$hw.$index");
                my $total = $used + $free;
				my $used_percent = ($used/$total)*100;

				$used_percent = sprintf("%.0f", $used_percent);

                my $descr = ESIA::SNMP::get($snmp,"$oid_cisco_ios_xe_physicaldescr.$hw");
                if( !defined($match) || $descr=~/$match/)
                {
                    $used/=(1024*1024);
                    $total/=(1024*1024);
                    $used = sprintf("%.0f",$used);
                    $total = sprintf("%.0f",$total);

                    my $state = 'UNKNOWN';
                    if( $used_percent < $warning )
					{
						$state = 'OK';
						$output .= "$state: $descr memory used $used/$total MB $used_percent% (< $warning%)\n";
					}
					elsif( $used_percent < $critical )
					{
						$state = 'WARNING';
						$output .= "$state: $descr memory used $used/$total MB $used_percent% (>= $warning%)\n";
					}
					else
					{
						$state = 'CRITICAL';
						$output .= "$state: $descr memory used $used/$total MB $used_percent% (>= $critical%)\n";
					}
                    $total_percent += $used_percent;
                    $nb++;
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
                }
            }
        }
        my $avg_percent = $total_percent / $nb;
        $graphdata .="avg_processor:$avg_percent%;";
        $output.="|$graphdata;warning:$warning;critical:$critical;\n";
    }
    return $found,$return,$output;
}

# CISCO-ENTITY-FRU-FAN
my $cefcFanTrayOperStatus = '1.3.6.1.4.1.9.9.117.1.4.1.1.1';
my %cefcFanTrayOperStatusType = (
	"1" => 'unknown',
	"2" => 'up',
	"3" => 'down',
	"4" => 'warning',
);

my %cefcFanTrayOperStatusMessage = (
	"1" => 'unknown',
	"2" => 'powered on',
	"3" => 'powered down',
	"4" => 'partial failure, need replacement as soon as possible.',
);

sub getCiscoEntityFruFan
{
	my($snmp) = @_;
	my $ret = 0;
	my $out = "";
	my $found = 0;

	my $result = $snmp->get_table(Baseoid => $cefcFanTrayOperStatus);
    if (defined($result))
    {
		$found = 1;
		foreach my $key ( sort keys %$result)
		{
			#on récupère l'index
			if($key=~ /([0-9]*)$/ )
			{
				my $index = $1;
				my $operStatus = $$result{"$key"};
				my $operStatusType = $cefcFanTrayOperStatusType{"$operStatus"};
				my $operStatusMessage = $cefcFanTrayOperStatusMessage{"$operStatus"};
				my $descr = ESIA::SNMP::get($snmp,"$oid_cisco_ios_xe_physicaldescr.$index");
				
				my $state = 'UNKNOWN';
				if( $operStatus == 2 )
				{
					$state = 'OK';
				}
				elsif( $operStatus == 3)
				{
					$state = 'CRITICAL';
				}
				elsif( $operStatus == 4 )
				{
					$state = 'WARNING';
				}
				$out .= "$state : $descr is $operStatusType ($operStatusMessage)\n";
				$ret = ESIA::PLUGIN::priority_state($ret,$ERRORS{$state});
			}
		}
	}
	return $found,$ret,$out;
}

# CISCO-ENTITY-FRU-POWER
my $cefcFRUPowerAdminStatus = '1.3.6.1.4.1.9.9.117.1.1.2.1.1';
my $cefcFRUPowerOperStatus = '1.3.6.1.4.1.9.9.117.1.1.2.1.2';

my %cefcFRUPowerAdminStatusType = (
	"1" => 'on',
	"2" => 'off',
	"3" => 'inlineAuto',
	"4" => 'inlineOn',
	"5" => 'powerCycle',
);

my %cefcFRUPowerOperStatusType = (
	"1" => 'offEnvOther',
	"2" => 'on',
	"3" => 'offAdmin',
	"4" => 'offDenied',
	"5" => 'offEnvPower',
	"6" => 'offEnvTemp',
	"7" => 'offEnvFan',
	"8" => 'failed',
	"9" => 'onButFanFail',
	"10" => 'offCooling',
	"11" => 'offConnectorRating',
	"12" => 'onButInlinePowerFail',
);

my %cefcFRUPowerOperStatusMessage = (
	"1" => 'FRU is powered off because of unknown problem',
	"2" => 'FRU is powered on',
	"3" => 'Administratively off',
	"4" => 'FRU is powered off because available system power is insufficient',
	"5" => "FRU is powered off because of power problem in the FRU.",
	"6" => 'FRU is powered off because of temperature problem.',
	"7" => 'FRU is powered off because of fan problems.',
	"8" => 'FRU is in failed state',
	"9" => 'FRU is on, but fan has failed',
	"10" => "FRU is powered off because of the system's insufficient cooling capacity",
	"11" => "FRU is powered off because of the system's connector rating exceeded.",
	"12" => 'The FRU on, but no inline power is being delivered as the data/inline power component of the FRU has failed."',
);

sub getCiscoEntityFruPower
{
	my($snmp) = @_;
	my $ret = 0;
	my $out = "";
	my $found = 0;

	my $result = $snmp->get_table(Baseoid => $cefcFRUPowerOperStatus);
    if (defined($result))
    {
		$found = 1;
		foreach my $key ( sort keys %$result)
		{
			#on récupère l'index
			if($key=~ /([0-9]*)$/ )
			{
				my $index = $1;
				my $operStatus = $$result{"$key"};
				my $operStatusType = $cefcFRUPowerOperStatusType{"$operStatus"};
				my $operStatusMessage = $cefcFRUPowerOperStatusMessage{"$operStatus"};

				my $descr = ESIA::SNMP::get($snmp,"$oid_cisco_ios_xe_physicaldescr.$index");
			
				my $adminStatus = ESIA::SNMP::get($snmp,"$cefcFRUPowerAdminStatus.$index");
				my $adminStatusType = $cefcFRUPowerAdminStatusType{"$adminStatus"};
				
				my $state = 'CRITICAL';
				if( $operStatus == 2 )
				{
					$state = 'OK';
				}
				elsif( $operStatus == 9 || $operStatus == 12 )
				{
					$state = 'WARNING';
				}
				$out .= "$state : $descr Operational status is $operStatusType ($operStatusMessage). Admin status is $adminStatusType\n";
				$ret = ESIA::PLUGIN::priority_state($ret,$ERRORS{$state});
			}
		}
	}
	return $found,$ret,$out;
}

# CISCO-ENTITY-SENSOR-MIB
sub getCiscoEntitySensorMib
{
	my($snmp,$type,$divisor) = @_;
    my $max_value = 0;
    my $min_value = -1;
	my $ret = 0;
	my $out = "";
	my $found = 0;
	my $result = $snmp->get_table(Baseoid => $oid_cisco_ios_xe_physicaldescr);
    if (defined($result))
    {
    	$found = 1;
		foreach my $key ( sort keys %$result)
		{
			#on récupère l'index
			if($key=~ /([0-9]*)$/ )
			{
				my $index =$1;
				my $descr =$$result{"$key"};
				if($descr=~/$type/)
				{
					my $Sensorstate = ESIA::SNMP::get($snmp,"$oid_cisco_ios_xe_status.$index");
					my $state = "OK";
					my $message ="operational";
					if($Sensorstate ==2)
					{
						$message ="unavailable";
						$state = "WARNING";
						$ret = ESIA::PLUGIN::priority_state($ret,$ERRORS{$state});
					}
					elsif($Sensorstate==3)
					{
						$message ="nonoperational";
						$state = "CRITICAL";
						$ret = ESIA::PLUGIN::priority_state($ret,$ERRORS{$state});
					}
					else
					{
						my $value = ESIA::SNMP::get($snmp,"$oid_cisco_ios_xe_value.$index") / $divisor;
                        			$value = sprintf("%.0f", $value);
						my $type =ESIA::SNMP::get($snmp,"$oid_cisco_ios_xe_type.$index");
						$message.=" $value ".$cisco_ios_xe_type_text[$type];
						my $snmp_thresh= $snmp->get_table(Baseoid => "$oid_cisco_ios_xe_ThresholdEval.$index");
						my $sensorstreshstate = 0;
						#on évalue les données des capteurs
						foreach my $index ( keys %$snmp_thresh)
						{
							my $treshState =$$snmp_thresh{"$index"};
							if($treshState != 2)
							{
								$state = "CRITICAL";
								$ret = ESIA::PLUGIN::priority_state($ret,$ERRORS{$state});
								$message.= " ==> Threshold activate!";
								last;
							}

						}
                        if ( $max_value < $value )
                        {
                            $max_value = $value;
                        }
                        if ( $min_value == -1 || $min_value > $value )
                        {
                            $min_value = $value;
                        }
					}
					$out .= "$state : $descr is $message \n";              
				}
			}
		}
        if( $max_value > 0)
        {
            my $graphdata .="max=$max_value;min=$min_value";
            $out.="|$graphdata;\n";
        }
	}
	return $found,$ret,$out;
}
# CISCO-ENTITY-SENSOR-MIB
sub getCiscoOldChassis
{
	my($snmp,$type) = @_;
	my $ret = 0;
	my $out = "";
	my $found = 0;
    my $result = $snmp->get_table(Baseoid => $oid_cisco_chassis_card_descr);
    if (defined($result))
    {
    	$found = 1;
        #Test des variable de chassis
        foreach my $key ( keys %$result)
        {
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $descr =$$result{"$key"};
		        if($descr=~/$type/)
				{
					my $resultcard = $snmp->get_request("$oid_cisco_chassis_card_state.$index");
		        	my $chassisStatus =   ESIA::SNMP::get($snmp,"$oid_cisco_chassis_card_state.$index");
                	my $state = $chassis_status{$chassisStatus};
                	$out .= "$state : $descr is ".$chassis_status_message{$chassisStatus}."\n";
                	$ret = priority_state($ret,$ERRORS{$state});
                }
            }
        }
    }
    return $found,$ret,$out;
}
# CISCO-ENTITY-SENSOR-MIB
sub getCiscoASR
{
	my($snmp,$type,$match) = @_;
	my $hostname = $snmp->hostname();
	my $filecache = $hostname."_ASRsensors.json";
	my $sensorsInformation;
	my $ret = 0;
	my $out = "";
	my $found = 0;
	if($match eq '' || !defined($match))
	{
		$match='.*';
	}
	my $cache = ESIA::CacheFile->new('/tmp/ciscoSensor/',$filecache,30);
	if($cache->isValid() == 1)
	{
		my $data = $cache->getData();
		$sensorsInformation = decode_json($data);
		$found = 1;
	}
	else
	{
		my $filelock = $hostname."_ASRsensors.lock";
		my $lock = ESIA::CacheFile->new('/tmp/ciscoSensor/',$filelock,40);
		my $locktime =0;
		if($lock->isValid() == 1)
		{
			$locktime = $lock->getData();
		}
		my $now = time();
		my $delta = $now - $locktime;
		if($delta > 1200)
		{
				# reload demandé
			$lock->writeData($now);
			my @arr;
			my $result = $snmp->get_table(Baseoid => $oid_cisco_asr_sensors);
			if (defined($result))
			{
				$found = 1;
				foreach my $key ( keys %$result)
				{
					#on récupère l'index
					if($key=~ /([0-9]*)$/ )
					{
						my %hash;
				        my $index =$1;
				        $hash{'index'} = $index;
				        $hash{'type_numeric'} = $$result{"$key"};
						$hash{'type'} = $ciscoAsrDataType{$$result{"$key"}};
						$hash{'entity_id'} = ESIA::SNMP::get($snmp,"$oid_cisco_asr_sensors_link_entity.$index");
						$hash{'name'} = ESIA::SNMP::get($snmp,"$oid_entity_name.".$index);
						$hash{'hardware'} = ESIA::SNMP::get($snmp,"$oid_entity_name.".$hash{'entity_id'});
						$hash{'name'}=~ s/^\s+|\s+$//g;
						$hash{'hardware'}=~ s/^\s+|\s+$//g;
						my $tmp =  ESIA::SNMP::get($snmp,"$oid_cisco_asr_sensors_prec.$index");
						if($tmp eq 'noSuchInstance')
						{
							$tmp=0;
						}
						$hash{'precision'} = $tmp;
						$tmp =  ESIA::SNMP::get($snmp,"$oid_cisco_asr_sensors_multi.$index.0");
						if($tmp eq 'noSuchInstance')
						{
							$tmp=9;
						}
						$hash{'multiplicator'} = $tmp;
						#print " $index $oid_entity_name.".$hash{'entity_id'}.".0 = ".$hash{'name'}."\n";exit(0);
						push(@arr,\%hash);
				    }
				}
			}
			my $json = encode_json \@arr;
			$cache->writeData($json);
			$sensorsInformation =  \@arr;
		}
		else
		{
			#Trace($debug,"There is another process that reload data.");
			my $data = $cache->getData();
			$sensorsInformation = decode_json($data);
			$found = 1;
		}
	}
	my $countsensor = 0;
	my @arr;
	foreach my $sensor ( @$sensorsInformation )
	{
		if($sensor->{type}=~/$type/ && $sensor->{name}=~/$match/)
		{
			my %hash;
			my $value = ESIA::SNMP::get($snmp,"$oid_cisco_asr_sensors_val.".$sensor->{index});
			my $state = 'OK';
			$countsensor++;
			if($sensor->{multiplicator} != 9)
			{
				my $exp = ($sensor->{multiplicator}-9)*3;
				my $digit = pow(10,$exp);
				$value = $value*$digit;
			}
			if($sensor->{precision} > 0)
			{
				$value /= (10*$sensor->{precision});
			}
			$hash{'index'} = $sensor->{index};
			$hash{'name'} = $sensor->{name};
			$hash{'value'} = $value;
			$hash{'hardware'} = $sensor->{hardware};
			push(@arr,\%hash);
		}
	}
    return \@arr;
}
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::Session(@ARGV);
my $match;
GetOptions(
	"V"   => \$opt_V,	 "version"    => \$opt_V,
	"h"   => \$opt_h,	 "help"       => \$opt_h,
	"T=s" => \$type,	 "Type=s" => \$type,
	"m=s" => \$match,	 "match=s" => \$match,
	"w=i" => \$warning,	 "warning=i" => \$warning,
	"c=i" => \$critical, "critical=i" => \$critical,
	"J" => \$json,	 "json" => \$json,
    "show-ignore" => \$showIgnored,
);

# -h & --help print help
if ($opt_h)
{
    #affiche l'aide par défaut
    ESIA::SNMP::print_snmp_usage($PROGNAME,'-T [fan|psu|temp] [options]');
    my @arr;
    push(@arr,["-T","--type","type of test [fan,cpu,unit]" ]);
    push(@arr,["-w","--warning","warning threshold max in (degrees for temp, percent for cpu)" ]);
    push(@arr,["-c","--critical","critical threshold max in (degrees for temp, percent for cpu)"]);
    push(@arr,["","--show-ignore","Print also components/values ignored"]);
    ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
    exit $ERRORS{'OK'};
}

# -V & --version print version
if ($opt_V)
{
    print "$PROGNAME version $PROGVERSION\n";
    exit $ERRORS{'OK'};
}

#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}

my $state = 'UNKNOWN';
my $output = "";

my $sysOid_value = ESIA::SNMP::get($snmp,$oid_sysObjectOID);
if( !defined( $sysOid_value ) )
{
	print ("UNKNOWN: SNMP no response. $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}

#FOR Cisco>otherEnterprises>cisco_sb
if( $sysOid_value =~ /1\.3\.6\.1\.4\.1\.9\.6\.1/ )
{
    # WORK WITH :.3.6.1.4.1.9.9.10.1.1.4.1.1.10
    # - SG300-28PP 28-Port Gigabit PoE+ Managed Switch
    # - SG350X-48P 48-Port Gigabit PoE Stackable Managed Switch
    if( $type eq 'cpu' )
    {
        if( $warning > $critical )
        {
            print "UNKNOWN : You must specify correct warning,critical tresholds.\n";
            exit $ERRORS{UNKNOWN};
        }
        my $load_time;
        my $load_value;

        if( !defined($match) || $match eq '5' || $match eq '5m' || $match eq '300s' )
        {
            $load_time = 5;
            $load_value = ESIA::SNMP::get($snmp,"$oid_cisco_sb_cpu_5m");

        }
        elsif( $match eq '1' || $match eq '1m' || $match eq '60s' )
        {
            $load_time = 1;
            $load_value = ESIA::SNMP::get($snmp,"$oid_cisco_sb_cpu_1m");
        }
        else
        {
            print "UNKNOWN : You must specify correct -m, --match for load ( 1,5 | 1m,5m | 60s,300s )\n";
            exit $ERRORS{UNKNOWN};
        }

        if( $load_value < $warning )
        {
            $state = 'OK';
            $output = "$state: CPU used $load_value% (load average $load_time minutes) (< $warning%)";
        }
        elsif( $load_value < $critical )
        {
            $state = 'WARNING';
            $output = "$state: CPU used $load_value% (load average $load_time minutes) (>= $warning%)";
        }
        else
        {
            $state = 'CRITICAL';
            $output = "$state: CPU used $load_value% (load average $load_time minutes) (>= $critical%)";
        }
        $load_value = sprintf("%d",$load_value); 
		$output.="|cpu=$load_value 5m;warning=$warning;critical=$critical;\n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
    }
    # WORK WITH :
    # - SG300-28PP 28-Port Gigabit PoE+ Managed Switch
    # - SG350X-48P 48-Port Gigabit PoE Stackable Managed Switch
    elsif($type eq 'mem')
    {
 		print "OK : Cisco switch $type not availaible on this cisco switch model\n";
		exit $ERRORS{OK};
    }
    elsif($type eq 'fan')
    {
        my $state = 'OK';
        my $result = $snmp->get_table( $oid_ciscoEnvMonFanStatusDesc);

        foreach my $key ( sort keys %$result)
        {
            if($key=~/\.([0-9]*)$/)
            {
                my $id = $1;
                my $description = $$result{"$key"};
                if( !defined($match) || $description =~ /$match/)
                {
                    my $state_int = ESIA::SNMP::get($snmp,"$oid_ciscoEnvMonFanState.$id");
                    my $state_desc = $ciscoEnvMonState{$state_int};
                    $state = $ciscoEnvMonStateToEsia{$state_int};

                    if( !$showIgnored && $state eq 'IGNORE')
                    {
                        last;
                    }

                    $output .= "$state: FAN $description state is $state_desc\n";
                    if( $state eq 'IGNORE' )
                    {
                        $state = 'OK';
                    }
                }
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
            }
        }
    }
    # WORK WITH :
    # - SG350X-48P 48-Port Gigabit PoE Stackable Managed Switch
    # NOT WORKING WITH :
    # - SG300-28PP 28-Port Gigabit PoE+ Managed Switch
    elsif($type eq 'psu')
    {
        my $state = 'OK';
        my $result = $snmp->get_table( $oid_ciscoEnvMonSupplyStatusDescr);

        foreach my $key ( sort keys %$result)
        {
            if($key=~/\.([0-9]*)$/)
            {
                my $id = $1;
                my $description = $$result{"$key"};
                if( !defined($match) || $description =~ /$match/)
                {
                    my $state_int = ESIA::SNMP::get($snmp,"$oid_ciscoEnvMonSupplyState.$id");
                    my $state_desc = $ciscoEnvMonState{$state_int};
                    $state = $ciscoEnvMonStateToEsia{$state_int};

                    if( !$showIgnored && $state eq 'IGNORE')
                    {
                        last;
                    }

                    $output .= "$state: PSU $description state is $state_desc\n";
                    if( $state eq 'IGNORE' )
                    {
                        $state = 'OK';
                    }
                }
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
            }
        }
		#OID no present on this model
		if($output eq "")
		{
			print "OK : Cisco switch $type not availaible on this cisco switch model\n";
			exit $ERRORS{OK};
		}
    }
    # WORK WITH :
    # - SG350X-48P 48-Port Gigabit PoE Stackable Managed Switch
    # NOT WORKING WITH :
    # - SG300-28PP 28-Port Gigabit PoE+ Managed Switch
    elsif($type eq 'temp')
    {
		if( $warning == 80 && $critical == 90)
		{
			$warning=60;
			$critical=70;
		}
        if( $warning > $critical )
        {
            print "UNKNOWN : You must specify correct warning,critical tresholds.\n";
            exit $ERRORS{UNKNOWN};
        }

        my $state = 'OK';
        my $result = $snmp->get_table( $oid_rlEnvFanDataTemp);

		my $nb = 0;
		my $min = 0;
		my $sum = 0;
		my $max = 0;

        foreach my $key ( sort keys %$result)
        {
            if($key=~/\.([0-9]*)$/)
            {
                my $id = $1;
                my $temp = $$result{"$key"};
                if( !defined($match) || "$id" eq $match )
                {
                    if( $temp == 0 )
                    {
                        if( !$showIgnored )
                        {
                            last;
                        }
                        else
                        {
                            $state = 'OK';
                            $output .= "$state: TEMP $id value $temp degrees";
                        }
                    }
                    else
                    {
                        if( $temp < $warning )
                        {
                            $state = 'OK';
                            $output .= "$state: TEMP $id value $temp degrees (< $warning)";
                        }
                        elsif( $temp < $critical )
                        {
                            $state = 'WARNING';
                            $output .= "$state: TEMP $id value $temp degrees (> $warning)";
                        }
                        else
                        {
                            $state = 'CRITICAL';
                            $output .= "$state: TEMP $id value $temp degrees (> $critical)";
                        }

						if( $nb == 0 )
						{
							$min = $sum = $max = $temp;
						}
						else{
							if( $min > $temp )
							{
								$min = $temp;
							}
							if( $max < $temp )
							{
								$max = $temp;
							}
							$sum+= $temp;
						}
						$nb++;
                    }
                    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
                }
				if( $nb != 0 )
				{
					my $avg = round($sum/$nb);
					$output.="|min=$min;temp=$avg;max=$max;warning=$warning;critical=$critical\n";
				}
            }
        }
		#OID no present on this model
		if($output eq "")
		{
			print "OK : Cisco switch $type not availaible on this cisco switch model\n";
			exit $ERRORS{OK};
		}
    }
   # WORK WITH :
	# - SG300-28PP 28-Port Gigabit PoE+ Managed Switch
    # - SG350X-48P 48-Port Gigabit PoE Stackable Managed Switch
    elsif($type eq 'flash')
    {
        if( $warning > $critical )
        {
            print "UNKNOWN : You must specify correct warning,critical tresholds.\n";
            exit $ERRORS{UNKNOWN};
        }
		if( defined($match) && $match ne "flash" )
		{
			print "UNKNOWN : Cisco switch $type status retrieval failed. don't use match or use -m, --match falsh for this model\n";
			exit $ERRORS{UNKNOWN};
		}

		my $mem_total = ESIA::SNMP::get($snmp,"$oid_cisco_sb_FileTotalSizeOfFlash");
		my $mem_free = ESIA::SNMP::get($snmp,"$oid_cisco_sb_FileFreeSizeOfFlash");


		if( $mem_total <= 0 || $mem_free < 0)
		{
			print "UNKNOWN : Cisco switch $type status retrieval failed.\n";
			exit $ERRORS{UNKNOWN};
		}

		my $mem_usage = $mem_total - $mem_free;

		my $mem_usage_per = ($mem_usage / $mem_total) * 10000;
		$mem_usage_per = round($mem_usage_per);
		$mem_usage_per/=100;


		my $mem_total_h = round($mem_total/1024);
		my $mem_usage_h = round($mem_usage/1024);
		my $mem_unit = 'KB';

		if( $mem_total_h > 10000 )
		{
			$mem_total_h = round($mem_total/1024);
			$mem_usage_h = round($mem_usage/1024);
			$mem_unit = 'MB';
		}

   		if( $mem_usage_per < $warning )
        {
            $state = 'OK';
            $output = "$state: FLASH usage $mem_usage_per% ($mem_usage_h/$mem_total_h $mem_unit) (< $warning%)";
        }
        elsif( $mem_usage_per < $critical )
        {
            $state = 'WARNING';
            $output = "$state: FLASH usage $mem_usage_per% ($mem_usage_h/$mem_total_h $mem_unit) (>= $warning%)";
        }
        else
        {
            $state = 'CRITICAL';
            $output = "$state: FLASH usage $mem_usage_per% ($mem_usage_h/$mem_total_h $mem_unit) (>= $critical%)";
        }
		$output.="|percent=$mem_usage_per;usage=$mem_usage;total=$mem_total;warning=$warning;critical=$critical;\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
    }
    else
    {
        print "UNKNOWN : Incorrect parameter -T, --type $type.\n";
        ESIA::SNMP::print_snmp_usage($PROGNAME,'-T [fan|psu|temp]');
        exit $ERRORS{UNKNOWN};
    }
}
#FOR CiscoProducts
elsif( $sysOid_value =~ /1\.3\.6\.1\.4\.1\.9\.1\.([0-9]*)/ )
{
	# WORK WITH :
	# - C2950 Software (C2950-I6Q4L2-M)
    # - C3750 Software (C3750-IPBASEK9-M)
    my $is_asr = 0;
    if($1 == 1017 ||  $1 == 1018 ||  $1 == 1036 ||  $1 == 1037)
    {
    	 $is_asr = 1;
    	# on est dans la gamme cisco ASR
		if($type eq 'mem')
		{
			$match='processor';
		}
    }
    if( $type eq 'cpu' )
    {
        my ($f,$ret,$out) = getCiscoProcessCPUUsage($snmp,$sysOid_value,$warning,$critical,$match);
        $return = $ret;
        $output = $out;
    }
    elsif( $type eq 'mem' )
    {
        if( !defined($match))
        {
        	$match='.*';
        }
        my $result = $snmp->get_table( $cisco_index);
        my $graphdata="";
        foreach my $key ( sort keys %$result)
        {
            if($key=~/\.([0-9]*)$/)
            {
                my $state = 'OK';
                my $id = $1;
                my $description = $$result{"$key"};
                if($description=~/$match/)
                {
					my $used = ESIA::SNMP::get($snmp,"$cisco_used.$id");
					my $free = ESIA::SNMP::get($snmp,"$cisco_free.$id");
					my $total = $used + $free;
					my $used_percent = ($used/$total)*100;
					$used_percent = sprintf("%.0f", $used_percent);
					if( $used_percent < $warning )
					{
						$state = 'OK';
						$output .= "$state: $description memory used $used_percent% (< $warning%)\n";
					}
					elsif( $used_percent < $critical )
					{
						$state = 'WARNING';
						$output .= "$state: $description memory used $used_percent% (>= $warning%)\n";
					}
					else
					{
						$state = 'CRITICAL';
						$output .= "$state: $description memory used $used_percent% (>= $critical%)\n";
					}
					$graphdata .="$description=$used_percent;";
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
                }
            }
        }
       	$output.="|$graphdata;warning=$warning;critical=$critical;\n";
    }
    elsif($type eq 'fan')
    {
    	#my $oid_cisco_asr_sensors_threshold='1.3.6.1.4.1.9.9.91.1.2.1.1.5';
		#my $oid_cisco_asr_sensors_status   ='1.3.6.1.4.1.9.9.91.1.1.1.1.5';
        if($is_asr == 1)
        {
        	my ($sensors) = getCiscoASR($snmp,'rpm|cmm');
        	my $nbrsensors=0;
        	my $errorsensors=0;
        	my $total_thresh_sensors=0;
        	my $nok_thresh_sensors=0;
        	my $out="";
        	my $outtresh="";
         	foreach my $sensor ( @$sensors )
			{
				$nbrsensors++;
				my $sensorsstatus =  ESIA::SNMP::get($snmp,"$oid_cisco_asr_sensors_status.".$sensor->{index});
				if( $sensorsstatus == 1 || $sensorsstatus == 2)
				{
					my $result = $snmp->get_table( $oid_cisco_asr_sensors_threshold.".".$sensor->{index});
					if (defined($result))
					{
						foreach my $key ( sort keys %$result)
						{
							my $state = $$result{"$key"};

							if($state == 1)
							{
								$nok_thresh_sensors++;
								$outtresh.="CRITICAL: ". $sensor->{name}.' on '.$sensor->{hardware}." threshold result is out of range.\n";
							}
							$total_thresh_sensors++;
						}
					}
				}
				else
				{
					$out.="CRITICAL: ". $sensor->{name}.' on '.$sensor->{hardware}." is broken\n";
					$errorsensors++;
				}

			}
			if($nbrsensors==0)
			{
				$output="OK: there are not fan sensors";
			}
			else
			{
				if($errorsensors > 0)
				{
					$output="CRITICAL:  $errorsensors/$nbrsensors fan sensors are nonoperational\n$out";
				}
				else
				{
					$output="OK: $nbrsensors fan sensors are operational\n";
				}
				if($nok_thresh_sensors>0)
				{
					$output.="CRITICAL:  $nok_thresh_sensors/$total_thresh_sensors fan tests are in error states\n$outtresh";
				}
				else
				{
					$output.="OK: $total_thresh_sensors fan check tests are in correct values\n";
				}
			}
        }
        else
        {
		    my $state = 'OK';
		    my $found = 0;
		    my $result = $snmp->get_table( $oid_ciscoFanDesc);
			if (defined($result))
			{
				#
				# CISCO-ENVMON-MIB
				#
				$found = 1;
				foreach my $key ( keys %$result)
				{
					#on récupère l'index
					if($key=~ /([0-9]*)$/ )
					{
						my $index =$1;
						my $descr =$$result{"$key"};
						my $fanstatus =  ESIA::SNMP::get($snmp,"$oid_ciscoFanStatut.$index");
						my $state = $envmon_status{$fanstatus};
						$output .= "$state : FAN $descr is ".$envmon_status_message{$fanstatus}."\n";
						$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});

					}
				}
			}
			else
			{
				#
				# Cisco CISCO-ENTITY-SENSOR-MIB
				#
				my ($f,$ret,$out) = getCiscoEntitySensorMib($snmp,'Fan|FAN|fan',1);
				if($f == 1)
				{
					$return = ESIA::PLUGIN::priority_state($return,$ret);
					$output=$out;
				}
				else
				{
					my ($f,$ret,$out) = getCiscoOldChassis($snmp,'Fan|FAN|fan');
					if($f == 1)
					{
						$return = ESIA::PLUGIN::priority_state($return,$ret);
						$output=$out;
					}
					else
					{
						print "OK : model not support fans status\n";
						$snmp->close;
						exit $ERRORS{"OK"};
					}
				}
			}
		}
    }
    elsif($type eq 'psu')
    {
        my $state = 'OK';
        if($is_asr == 1)
        {
        	my $nbrsensors=0;
        	my $errorsensors=0;
        	my $total_thresh_sensors=0;
        	my $nok_thresh_sensors=0;
        	my $out="";
        	my $outtresh="";
        	my ($sensors) = getCiscoASR($snmp,'voltsAC|voltsDC|amperes|watts|hertz');
         	foreach my $sensor ( @$sensors )
			{
				$nbrsensors++;
				my $sensorsstatus =  ESIA::SNMP::get($snmp,"$oid_cisco_asr_sensors_status.".$sensor->{index});
				if( $sensorsstatus == 1 || $sensorsstatus == 2)
				{
					my $result = $snmp->get_table( $oid_cisco_asr_sensors_threshold.".".$sensor->{index});
					if (defined($result))
					{
						foreach my $key ( sort keys %$result)
						{
							my $state = $$result{"$key"};

							if($state == 1)
							{
								$nok_thresh_sensors++;
								$outtresh.="CRITICAL: ". $sensor->{name}.' on '.$sensor->{hardware}." threshold result is out of range.\n";
							}
							$total_thresh_sensors++;
						}
					}
				}
				else
				{
					$out.="CRITICAL: ". $sensor->{name}.' on '.$sensor->{hardware}." is broken\n";
					$errorsensors++;
				}

			}
			if($errorsensors > 0)
			{
				$output="CRITICAL:  $errorsensors/$nbrsensors PSU sensors are nonoperational\n$out";
			}
			else
			{
				$output="OK: $nbrsensors PSU sensors are operational\n";
			}
			if($nok_thresh_sensors>0)
			{
				$output.="CRITICAL:  $nok_thresh_sensors/$total_thresh_sensors PSU tests are in error states\n$outtresh";
			}
			else
			{
				$output.="OK: $total_thresh_sensors PSU check tests are in correct values\n";
			}
        }
        else
        {
		    my $found = 0;
		    my $result = $snmp->get_table( $oid_ciscoPowerDesc);
			if (defined($result))
			{
				#
				# CISCO-ENVMON-MIB
				#
				$found = 1;
				foreach my $key ( keys %$result)
				{
					#on récupère l'index
					if($key=~ /([0-9]*)$/ )
					{
						my $index =$1;
						my $descr =$$result{"$key"};
						my $powerstatus =  ESIA::SNMP::get($snmp,"$oid_ciscoPowerStatut.$index");
		        		my $state = $envmon_status{$powerstatus};
						$output .= "$state : PSU $descr is ".$envmon_status_message{$powerstatus}."\n";
						$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});

					}
				}
			}
			else
			{
				#
				# Cisco CISCO-ENTITY-SENSOR-MIB
				#
				my ($f,$ret,$out) = getCiscoEntitySensorMib($snmp,'PSU|psu',1);
				if($f == 1)
				{
					$return = ESIA::PLUGIN::priority_state($return,$ret);
					$output=$out;
				}
				else
				{
					my ($f,$ret,$out) = getCiscoOldChassis($snmp,'PSU|psu');
					if($f == 1 && $out ne "")
					{
						$return = ESIA::PLUGIN::priority_state($return,$ret);
						$output=$out;
					}
					else
					{
						print "OK : model not support PSU status\n";
						$snmp->close;
						exit $ERRORS{"OK"};
					}
				}
			}
		}
    }
    elsif($type eq 'temp')
    {
		if( $warning == 80 && $critical == 90)
		{
			$warning=60;
			$critical=70;
		}
        if( $warning > $critical )
        {
            print "UNKNOWN : You must specify correct warning,critical tresholds.\n";
            exit $ERRORS{UNKNOWN};
        }
        if($is_asr == 1)
        {
        	my ($sensors) = getCiscoASR($snmp,'celsius','Hot');
        	my $nbrsensor=0;
        	my $valuemin=0;
        	my $valuemax=0;
        	my $valueavg=0;
        	my $sensorerror=0;
        	my $out="";
        	foreach my $sensor ( @$sensors )
			{
				$nbrsensor++;
				if($sensor->{value} < $valuemin)
				{
					$valuemin= $sensor->{value};
				}
				if($sensor->{value} > $valuemax)
				{
					$valuemax= $sensor->{value};
				}
				$valueavg = $valueavg+$sensor->{value};
				if($sensor->{value} > $critical)
				{
					$sensorerror++;
					$out.="CRITICAL: ".$sensor->{name}." on ".$sensor->{hardware}." temperature is very high ".$sensor->{value}."°C > $critical°C\n";
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{'CRITICAL'});
				}
				elsif($sensor->{value} > $warning)
				{
					$sensorerror++;
					$out.="WARNING: ".$sensor->{name}." on ".$sensor->{hardware}." temperature is high ".$sensor->{value}."°C > $warning°C\n";
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{'WARNING'});
				}
			}
			$valueavg=sprintf("%.1f",($valueavg/$nbrsensor));
			if($return == $ERRORS{'OK'})
			{
				$output= "OK: all ($nbrsensor) temperature sensors is OK state,";
			}
			elsif($return == $ERRORS{'CRITICAL'})
			{
				$output= "CRITICAL: there are $sensorerror temperature sensors is error state,";
			}
			elsif($return == $ERRORS{'WARNING'})
			{
				$output= "WARNING: there are $sensorerror temperature sensors is error state,";
			}
			$output.= "min=$valuemin° max=$valuemax°. average=$valueavg (w:$warning,c:$critical)\n$out";
        }
        else
        {
		    my $state = 'OK';
		    my $found = 0;
		    my $result = $snmp->get_table( $oid_ciscoTempDesc);
			if (defined($result))
			{
				#
				# CISCO-ENVMON-MIB
				#
				$found = 1;
				foreach my $key ( keys %$result)
				{
					#on récupère l'index
					if($key=~ /([0-9]*)$/ )
					{
						my $index =$1;
						my $descr =$$result{"$key"};
						my $tempstatus =  ESIA::SNMP::get($snmp,"$oid_ciscoTempStatut.$index");
						if($tempstatus ne 'noSuchObject')
						{
							my $tempval =  ESIA::SNMP::get($snmp,"$oid_ciscoTempValue.$index");
						    my $state = $envmon_status{$tempstatus};
						    $output .="$state : $descr is ".$envmon_status_message{$tempstatus}.", $tempval\n";
							$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
						}

					}
				}
			}
			else
			{
				#
				# Cisco CISCO-ENTITY-SENSOR-MIB
				#
				my ($f,$ret,$out) = getCiscoEntitySensorMib($snmp,'Temp|temp',1);
				if($f == 1 && $out ne "")
				{
					$return = ESIA::PLUGIN::priority_state($return,$ret);
					$output=$out;
				}
				else
				{
					my ($f,$ret,$out) = getCiscoOldChassis($snmp,'Temp|temp');
					if($f == 1)
					{
						$return = ESIA::PLUGIN::priority_state($return,$ret);
						$output=$out;
					}
					else
					{
						print "OK : model not support temperature status\n";
						$snmp->close;
						exit $ERRORS{"OK"};
					}
				}
			}
		}
    }
    elsif($type eq 'flash')
    {
        if( !defined($match))
        {
        	$match='.*';
        }
		if( $warning == 80 && $critical == 90)
		{
			$warning=95;
			$critical=98;
		}
        my $result = $snmp->get_table( $oid_cisco_flash_name_partition);
        my $graphdata="";
        foreach my $key ( keys %$result)
        {
        	$state="OK";
			if($key=~ /3.6.1.4.1.9.9.10.1.1.4.1.1.10.([0-9\.]*)$/ )
			{
				my $index =$1;
				my $descr =$$result{"$key"};
				if($descr=~/$match/)
                {
					my $size = ESIA::SNMP::get($snmp,"$oid_cisco_flash_extended_size.$index");
					my $free =0;
					if($size eq 'noSuchObject')
					{
						$size =  ESIA::SNMP::get($snmp,"$oid_cisco_flash_size.$index");
						$free =  ESIA::SNMP::get($snmp,"$oid_cisco_flash_free.$index");
					}
					else
					{
						$free =  ESIA::SNMP::get($snmp,"$oid_cisco_flash_extended_free.$index");
					}
					my $used = $size - $free;
					my $percent = ($used/$size)*100;
					$percent = sprintf("%.0f", $percent);
					my $message = " < $warning%";
					if($percent < $warning)
					{
						$state="OK";
					}
					elsif($percent < $critical)
					{
						$state="WARNING";
						$message = " > $warning%";
					}
					else
					{
						$state="CRITICAL";
						$message = " > $critical%";
					}
					$output .="$state : $descr use $percent% $message\n";
					$graphdata="percent=$percent;usage=$used;total=$size";
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
				}
			}
        }
        if($output eq "")
        {
        	$output .="UNKNOWN : flash $match not found\n";
        	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
        }
        $output.="|$graphdata;warning=$warning;critical=$critical;\n";
    }
    elsif($type eq 'inventory')
    {
     	my ($out) = ESIA::INVENTORY::getDefaultChassisInventoryMib($snmp,$ESIA::INVENTORY::ARRAY);
        foreach my $line ( @$out )
		{
			if($line->{'type'} eq 'chassis')
			{
				$line->{'manufacturer'}='Cisco Systems';
				if($is_asr == 1)
        		{
					my $version = ESIA::SNMP::get($snmp,"$oid_cisco_asr_version_os.".$line->{'index'});
					if(defined($version))
					{
						$line->{'firmwareVersion'}=$version;
					}
				}
			}
		}
		if($json)
		{
			$output = encode_json(\@$out);
		}
		else
		{
			$output="";
            foreach my $line(@$out)
            {
                my $hash = \%$line;
                $output.= "name: ".$hash->{name}.", manufacturer: ".$hash->{manufacturer}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
                $output.=", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
            }
		}
    }
}

#FOR CiscoNexus
elsif( $sysOid_value =~ /1\.3\.6\.1\.4\.1\.9\.12\.3\.1\.3/ )
{
    if( $type eq 'cpu' )
    {
        my ($f,$ret,$out) = getCiscoProcessCPUUsage($snmp,$sysOid_value,$warning,$critical,$match);
        if( $f == 1 )
        {
            $return = $ret;
            $output = $out;
        }
    }
    elsif( $type eq 'mem')
    {
        my ($f,$ret,$out) = getCiscoEnhancedMemPool($snmp,$warning,$critical,$match);
        if( $f == 1 )
        {
            $return = $ret;
            $output = $out;
        }
    }
    elsif( $type eq 'fan')
    {
        my ($f,$ret,$out) = getCiscoEntityFruFan($snmp);
        if( $f == 1 )
        {
            $return = $ret;
            $output = $out;
        }
    }
    elsif( $type eq 'psu')
    {
        my ($f,$ret,$out) = getCiscoEntityFruPower($snmp);
        if( $f == 1 )
        {
            $return = $ret;
            $output = $out;
        }
    }
    elsif( $type eq 'temp')
    {
 	    my $state = 'OK';
		my $found = 0;
        $snmp->max_msg_size(16384);
		
		#
		# Cisco CISCO-ENTITY-SENSOR-MIB
		#
		my ($f,$ret,$out) = getCiscoEntitySensorMib($snmp,'Temp|temp',1000);
		if($f == 1 && $out ne "")
		{
			$return = ESIA::PLUGIN::priority_state($return,$ret);
			$output=$out;
		}
    }
    elsif( $type eq 'flash')
    {
     if( !defined($match))
        {
        	$match='.*';
        }
		if( $warning == 80 && $critical == 90)
		{
			$warning=95;
			$critical=98;
		}
        my $result = $snmp->get_table( $oid_cisco_flash_name_partition);
        my $graphdata="";
        my $max_percent = 0;
        my $max_used = 0;
        my $max_size=0 ; 
        foreach my $key ( sort keys %$result)
        {
        	$state="OK";
			if($key=~ /3.6.1.4.1.9.9.10.1.1.4.1.1.10.([0-9\.]*)$/ )
			{
				my $index =$1;
				my $descr =$$result{"$key"};
				if($descr=~/$match/)
                {
					my $size = ESIA::SNMP::get($snmp,"$oid_cisco_flash_extended_size.$index");
					my $free =0;
					if($size eq 'noSuchObject')
					{
						$size =  ESIA::SNMP::get($snmp,"$oid_cisco_flash_size.$index");
						$free =  ESIA::SNMP::get($snmp,"$oid_cisco_flash_free.$index");
					}
					else
					{
						$free =  ESIA::SNMP::get($snmp,"$oid_cisco_flash_extended_free.$index");
					}
					my $used = $size - $free;
					my $percent = ($used/$size)*100;
					$percent = sprintf("%.0f", $percent);
					my $message = " < $warning%";
					if($percent < $warning)
					{
						$state="OK";
					}
					elsif($percent < $critical)
					{
						$state="WARNING";
						$message = " > $warning%";
					}
					else
					{
						$state="CRITICAL";
						$message = " > $critical%";
					}
                    if( $max_percent < $percent)
                    {
                        $max_percent = $percent;
                        $max_used = $used;
                        $max_size = $size;
                    }
					$output .="$state : $descr use $percent% $message\n";
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
				}
			}
        }
        if($output eq "")
        {
        	$output .="UNKNOWN : flash $match not found\n";
        	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
        }
        $graphdata="percent=$max_percent;usage=$max_used;total=$max_size";
        $output.="|$graphdata;warning=$warning;critical=$critical;\n";
    }
    elsif( $type eq 'inventory')
    {
     	my ($out) = ESIA::INVENTORY::getDefaultChassisInventoryMib($snmp,$ESIA::INVENTORY::ARRAY);
        foreach my $line ( @$out )
		{
			if($line->{'type'} eq 'chassis')
			{
				$line->{'manufacturer'}='Cisco Systems';

				my $version = ESIA::SNMP::get($snmp,"$oid_cisco_asr_version_os.".$line->{'index'});
				if(defined($version))
				{
					$line->{'firmwareVersion'}=$version;
				}
			}
		}

	    if($json)
	    {
		    $output = encode_json(\@$out);
	    }
	    else
	    {
		    $output="";
            foreach my $line(@$out)
            {
                my $hash = \%$line;
                $output.= "name: ".$hash->{name}.", manufacturer: ".$hash->{manufacturer}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
                $output.=", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
            }
	    }
    }
}
else
{
    print "UNKNOWN : Device type not implemented for sysobject id $sysOid_value\n";
    exit $ERRORS{UNKNOWN};
}

if($output eq "")
{
    print "UNKNOWN : Cisco switch $type status retrieval failed.\n";
    exit $ERRORS{UNKNOWN};
}
else
{
    print $output;
    exit $return;
}
