#!/usr/bin/perl
use strict;
use warnings;
use lib "/usr/local/esia/plugins/lib/perl"  ;
#use lib "/root/libperl/"  ;
use vars qw($PROGNAME);
use Net::SNMP;
use Getopt::Long;
use ESIA::SNMP;
use ESIA::PLUGIN;
use ESIA::CacheFile;
use JSON;
#nom du programme & description
my $PROGNAME = "check_snmp_cisco_switch.pl";
my $PROGDESC = "Cisco switches by SNMP";
my $PROGVERSION = "1.0";
#Chargement du timer par défaut et des erreurs
# a installer apt install libmath-round-perl
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

ESIA::PLUGIN::arm_end_signal($TIMEOUT);

Getopt::Long::Configure( "bundling" );
my ($opt_h,$opt_V);
my $path='/tmp/meraki';
my $warning=80;
my $critical=90;
my $return =0;
my $type='alive';
my ($debug,$json);

my $oid_sysObjectOID= "1.3.6.1.2.1.1.2.0";
my $oid_meraki_name ="1.3.6.1.4.1.29671.1.1.4.1.2";
my $oid_meraki_serial ="1.3.6.1.4.1.29671.1.1.4.1.8";
my $oid_meraki_product ="1.3.6.1.4.1.29671.1.1.4.1.9";
my $oid_meraki_description ="1.3.6.1.4.1.29671.1.1.4.1.10";
my $oid_meraki_status ="1.3.6.1.4.1.29671.1.1.4.1.3";
my $oid_meraki_clientcount ="1.3.6.1.4.1.29671.1.1.4.1.5";
my $oid_meraki_lastConnected ="1.3.6.1.4.1.29671.1.1.4.1.4";
sub Trace
{
	 my($debug,$message) = @_;
	 if($debug)
	 {
	 	print "DEBUG: $message\n";
	 }
}

#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::Session(@ARGV);
my $match;
GetOptions(
	"V"   => \$opt_V,	 "version"    => \$opt_V,
	"h"   => \$opt_h,	 "help"       => \$opt_h,
	"T=s" => \$type,	 "Type=s" => \$type,
	"m=s" => \$match,	 "match=s" => \$match,
	"w=i" => \$warning,	 "warning=i" => \$warning,
	"c=i" => \$critical, "critical=i" => \$critical,
	"J"   => \$json,"json"     => \$json,
	"D"   => \$debug,"debug"     => \$debug,
);

# -h & --help print help
if ($opt_h)
{
    #affiche l'aide par défaut
    ESIA::SNMP::print_snmp_usage($PROGNAME,'-T [alive|client|inventory] [options]');
    my @arr;
    push(@arr,["-T","--type","type of test [alive,client,inventory]" ]);
    push(@arr,["-m","--match","name of the switch" ]);
    push(@arr,["-w","--warning","warning threshold max in (degrees for temp, percent for cpu)" ]);
    push(@arr,["-c","--critical","critical threshold max in (degrees for temp, percent for cpu)"]);
	push(@arr,["-D","--debug","enable Debug mode" ]);
    ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
    exit $ERRORS{'OK'};
}

# -V & --version print version
if ($opt_V)
{
    print "$PROGNAME version $PROGVERSION\n";
    exit $ERRORS{'OK'};
}

#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
my $filename = $snmp->hostname();
if($snmp->version() == 1 || $snmp->version() == 2)
{
	$filename = $snmp->security()->community();
}

my $sysOid_value = ESIA::SNMP::get($snmp,$oid_sysObjectOID);
if( !defined( $sysOid_value ) )
{
	print ("UNKNOWN: SNMP no response. $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
my $deviceInfo;
#préparation fichier cache
Trace($debug,"LOAD cache meraki mib");
my $cache = ESIA::CacheFile->new($path,$filename.'_device.json',15);
if($cache->isValid() == 1)
{
    my $data = $cache->getData();
    $deviceInfo = decode_json($data);
}
else
{
    Trace($debug,"create cache meraki mib");
	my $res = $snmp->get_table(Baseoid =>$oid_meraki_name);
    my @arr;
    my %hash_inter;
    foreach my $key ( keys %$res)
	{
		if($key =~ /3\.6\.1\.4\.1\.29671\.1\.1\.4\.1\.2\.([0-9\.]*)$/)
		{
			my %hash;
			my $index = $1;
			$hash{'name'}=$$res{"$key"};
			$hash{'index'}=$index;
			$hash{'serial'} = ESIA::SNMP::get($snmp,"$oid_meraki_serial.$index");
			$hash{'product'} = ESIA::SNMP::get($snmp,"$oid_meraki_product.$index");
			$hash{'description'} = ESIA::SNMP::get($snmp,"$oid_meraki_description.$index");
			push(@arr,\%hash);
		}
	}
	$hash_inter{'device'}=\@arr;
	my $json = encode_json(\%hash_inter);
	$cache->writeData($json);
	$deviceInfo =  \%hash_inter;
}
my $state = 'UNKNOWN';
my $arr = $deviceInfo->{'device'};
my $name;
my $index;
my $serial;
my $product;
my $description;
Trace($debug,"search in $match");
foreach my $key ( @$arr )
{
    my $found = 0;
	Trace($debug,'Test '.$key->{'name'}.' is equal at '.$match );
	if($key->{'name'} eq $match)
	{
		$found = 1;
	}
	if($found == 1)
	{
		$name= $key->{'name'};
		$index=$key->{'index'};
		$serial=$key->{'serial'};
		$product=$key->{'product'};
		$description=$key->{'description'};
		last;
	}
}
if (!defined($index))
{
	print ("UNKNOWN: $match name not define by -m parameter or device not found\n");
	exit $ERRORS{'UNKNOWN'};
}
my $output = "";
if($type eq 'alive')
{
	Trace($debug,"SNMP Request on  $oid_meraki_status.$index");
	my $status = ESIA::SNMP::get($snmp,"$oid_meraki_status.$index");
	my $message ="";
	if($status == 1)
	{
		$state='OK';
		$message ="online";
	}
	else
	{
		$state='CRITICAL';
		$message ="offline";
	}
	$output = "$state: $name (sn: $serial,product $product) is $message.\n";
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});

}
elsif($type eq 'client')
{
	Trace($debug,"SNMP Request on  $oid_meraki_status.$index");
	my $client = ESIA::SNMP::get($snmp,"$oid_meraki_clientcount.$index");
	$state='OK';
	my $message ="";
	if($client < $warning)
	{
		$state='OK';
		$message ="< $warning";
	}
	elsif($client < $critical)
	{
		$state='WARNING';
		$message ="> $warning";
	}
	else
	{
		$state='CRITICAL';
		$message ="> $critical";
	}
	$output = "$state: $name has $client client connected $message|client=$client \n";
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});

}
elsif($type eq 'inventory')
{
	my @output;
	my %hash = (
			'name' => $name,
			'description'=>"$description",
			'manufacturer'=>'cisco',
			'model' => "$product",
			'type'=>'CHASSIS',
			'serial' => $serial,
			'hardwareVersion' => '',
			'firmwareVersion' => '',
	);
	push(@output, \%hash);
	if($json)
	{
		my $out = encode_json(\@output);
		print $out;
	}
	else
	{
		my $size = @output;
		if($size != 0)
		{
			foreach my $line ( @output)
			{

				my $hash = \%$line;
				print "name: ".$hash->{name}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
				print ", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
			}
			exit 0;
		}
	}
}
if($output eq "")
{
    print "UNKNOWN : Meraki $type status retrieval failed.\n";
    exit $ERRORS{UNKNOWN};
}
else
{
    print $output;
    exit $return;
}
