#!/usr/bin/perl -w
############################## check_snmp_sophos.pl ########
my $Version='1.0';
# Date : 19 février 2019
# Author  : Nicolas Biersart
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;
use JSON;
my $PROGNAME='check_snmp_paloalto.pl';
my $PROGDESC = "Check Palo Alto firewall CPU, ram and environnement sensors";
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

my $json;
my $return=0;
my ($opt_h,$opt_V);
my $warning='';
my $critical='';
my $type = 'cpu';
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";
ESIA::PLUGIN::arm_end_signal($TIMEOUT);

#CPU
my $oid_paloalto_cpu_core_load 	= "1.3.6.1.2.1.25.3.3.1.2"; #
# MEM & Disk
my $oid_paloalto_descr = '1.3.6.1.2.1.25.2.3.1.3';
my $oid_paloalto_size = '1.3.6.1.2.1.25.2.3.1.5';
my $oid_paloalto_used = '1.3.6.1.2.1.25.2.3.1.6';
my $oid_paloalto_alloc_units= '1.3.6.1.2.1.25.2.3.1.4';

#HA
my $oid_paloalto_hastate		='1.3.6.1.4.1.25461.2.1.2.1.11.0'; #Current high-availability state. => active is OK
my $oid_paloalto_peerstate		='1.3.6.1.4.1.25461.2.1.2.1.12.0';
my $oid_paloalto_configMode		='1.3.6.1.4.1.25461.2.1.2.1.13.0'; #disabled, active-passive, or active-active
#session
my $oid_paloalto_session			='1.3.6.1.4.1.25461.2.1.2.3.1.0';
my $oid_paloalto_session_max		='1.3.6.1.4.1.25461.2.1.2.3.2.0';
my $oid_paloalto_session_active		='1.3.6.1.4.1.25461.2.1.2.3.3.0';
#inventory
my $oid_paloalto_chassis		='1.3.6.1.4.1.25461.2.1.2.2.1.0';
my $oid_paloalto_firmware		='1.3.6.1.4.1.25461.2.1.2.1.1.0';
my $oid_paloalto_hw_version		='1.3.6.1.4.1.25461.2.1.2.1.2.0';
my $oid_paloalto_serial			='1.3.6.1.4.1.25461.2.1.2.1.3.0';
my $oid_paloalto_hw_sensors		='1.3.6.1.2.1.99.1.1.1.1';
my $oid_paloalto_hw_sensors_desc='1.3.6.1.2.1.47.1.1.1.1.7';
my $oid_paloalto_hw_value		='1.3.6.1.2.1.99.1.1.1.4';
my $oid_paloalto_hw_value_unit	='1.3.6.1.2.1.99.1.1.1.6';
my $oid_paloalto_hw_state		='1.3.6.1.2.1.99.1.1.1.5';
my %oid_paloalto_hw_state_text = ( "1"  => "ok","2"  => "unavailable","3"  => "nonoperational");
my %oid_paloalto_hw_state_text_state = ( "1"  => "OK","2"  => "UNKNOWN","3"  => "CRITICAL");

#
#MAIN
#
&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);

GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
    "T=s" => \$type,"type=s" => \$type,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	"J" => \$json,	 "json" => \$json,
);
# -h & --help print help
if ($opt_h)
{
#affiche l'aide par défaut
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-T state');
	my @arr;
	push(@arr,["-T","--type","type of test [cpu,mem,disk,env,ha,session,inventory] (default: cpu)" ]);
	push(@arr,["-w","--warning","warning threshold in percent" ]);
	push(@arr,["-c","--critical","critical threshold in percent"]);
	push(@arr,["-j","--json","json output for inventory -T inventory required"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
#on interroge en SNMP pour récupérer les informations
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if($Sysoid_value==-1)
{
	print ("UNKNOWN: Host not SNMP response $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
# test des paramètre par défaut
if($warning eq '' )
{
    if($type eq 'cpu' || $type eq 'mem' || $type eq 'disk')
    {
        $warning=80;
    }
    else
    {
    	$warning=55;
    }
}
if($critical eq '')
{
    if($type eq 'cpu'|| $type eq 'mem')
    {
        $critical=80;
    }
    else
    {
    	$critical=65;
    }
}
#
#Palo Alto Firewall
#

if($type eq 'cpu')
{
	my $state = 'OK';
	my $result = $snmp->get_table(Baseoid => $oid_paloalto_cpu_core_load);
	my $number=0;
	my $cpu_load=0;
	foreach my $key ( keys %$result)
	{
		$cpu_load = $$result{"$key"} + $cpu_load;
		$number++;
	}
	if($number !=0)
	{
		$cpu_load=$cpu_load/$number;
	}
    my $message="(< $warning)";
    if($cpu_load >= $critical)
    {
    	$state='CRITICAL';
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
		$message="(> $critical)";
	}
	elsif($cpu_load >= $warning)
	{
		$state='WARNING';
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
		$message="(> $warning)";
	}
	printf("%s : CPU used %.1f%% %s|cpu=%.0f",$state,$cpu_load,$message,$cpu_load);
}
elsif($type eq 'mem' || $type eq 'disk')
{
	my $match="notfound";
	if($type eq 'mem')
	{
		$match="Slot-1 Management Memory";
	}
	else
	{
		$match="Slot-1 Management Root Partition";
	}
	my $flag_match=0;
	my $result = $snmp->get_table( $oid_paloalto_descr);
	my $graphdata="";
	foreach my $key ( keys %$result)
	{
	    my $name = $$result{$key};
    	if($name =~/$match/)
    	{
        	$flag_match=1;
			if($key=~/\.([0-9]*)$/)
			{
            	my $id = $1;
            	my $used = ESIA::SNMP::get($snmp,"$oid_paloalto_used.$id");
            	my $size = ESIA::SNMP::get($snmp,"$oid_paloalto_size.$id");
            	my $unit = ESIA::SNMP::get($snmp,"$oid_paloalto_alloc_units.$id");
		        my $percent_used = 0;
		        if($size != 0)
		        {
		            $percent_used = ($used/$size)*100;
		        }
		        $percent_used = sprintf("%.0f",$percent_used);
		        my $used_byte = $used * $unit;
		        my $size_byte = $size * $unit;
		        my $used_MB =sprintf("%.0f",$used_byte/1024/1024);
		        my $size_MB =sprintf("%.0f",$size_byte/1024/1024);
				my $state = "OK";
				my $message = "< $warning%";
				if($percent_used < $warning)
				{
					$state = "OK";
				}
				elsif($percent_used < $critical)
				{
					$state = "WARNING";
					$message = "> $warning%";
				}
				else
				{
					$state = "CRITICAL";
					$message = "> $critical%";
				}
		        print "$state: $name: ".$percent_used."%"."used($used_MB"."MB/$size_MB"."MB) $message\n";
		        $graphdata="used=$percent_used;usedByte=$used_byte;totalByte=$size_byte";
		        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
            }
		}
	}
	if($flag_match==0)
	{
		print"UNKNOWN: partition $match not found\n";
		$return = $ERRORS{'UNKNOWN'};
	}
	else
	{
		print"|$graphdata";
	}
}
elsif($type eq 'ha')
{
	my $state = 'OK';
	my $ha_enable = ESIA::SNMP::get($snmp,"$oid_paloalto_configMode");
	if($ha_enable ne 'disabled')
	{
		print "$state : HA is $ha_enable mode\n";
		my $ha_state = ESIA::SNMP::get($snmp,"$oid_paloalto_hastate");
		my $ha_peer = ESIA::SNMP::get($snmp,"$oid_paloalto_peerstate");
		if($ha_state eq 'active')
		{
			$state='OK';
			if($ha_enable eq 'active-passive')
			{
				if($ha_peer ne 'passive')
				{
					#le slave n'est pas en mode passif
					$state='WARNING';
				}
			}
			else
			{
				#le cluster est en mode actif-actif
				if($ha_peer ne 'active')
				{
					#le slave n'est pas en mode active
					$state='WARNING';
				}
			}
		}
		else
		{
			$state='CRITICAL';
		}
		print "$state : HA state is ".$ha_state." and peer is in ".$ha_peer." mode\n";
	}
	else
	{
		print "$state : HA is not configured";
	}
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
}
elsif($type eq 'env')
{
	my $result = $snmp->get_table( $oid_paloalto_hw_sensors);
	my $graphdata="";
	my $temp_avg=0;
	my $nbr_sensors=0;
	foreach my $key ( keys %$result)
	{
		my $type = $$result{$key};
    	if($type == 8)
    	{
		    if($key =~ /\.([0-9]*)$/)
			{
				my $id = $1;
				my $state='OK';
				my $descr = ESIA::SNMP::get($snmp,"$oid_paloalto_hw_sensors_desc.$id");
				my $value = ESIA::SNMP::get($snmp,"$oid_paloalto_hw_value.$id");
				my $unit = ESIA::SNMP::get($snmp,"$oid_paloalto_hw_value_unit.$id");
				my $status = ESIA::SNMP::get($snmp,"$oid_paloalto_hw_state.$id");
				$state =$oid_paloalto_hw_state_text_state{$status};
				$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
				$nbr_sensors++;
				$temp_avg=$temp_avg+$value;
				print "$state : $descr sensor is ".$oid_paloalto_hw_state_text{$status}.". Temperature is $value $unit\n";
			}
    	}
	}
	my $state='OK';
	if($nbr_sensors != 0)
	{
		$temp_avg = $temp_avg / $nbr_sensors;
		$temp_avg = sprintf("%.1f",$temp_avg);
		my $message = "(< $warning C)";
		if($temp_avg > $critical)
		{
			$state='CRITICAL';
			$message = "(> $critical C)";
		}
		elsif($temp_avg > $warning)
		{
			$state='WARNING';
			$message = "(> $warning C)";
		}
		print "$state : average temperature is $temp_avg Celsius $message|temp=$temp_avg\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	}
}

elsif($type eq 'session')
{
	my $state = 'OK';
	my $used = ESIA::SNMP::get($snmp,"$oid_paloalto_session");
	my $max = ESIA::SNMP::get($snmp,"$oid_paloalto_session_max");
	my $active = ESIA::SNMP::get($snmp,"$oid_paloalto_session_active");
	my $message="(< $warning%)";
	if($used >= $critical)
	{
		$state='CRITICAL';
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
		$message="(> $critical%)";
	}
	elsif($used >= $warning)
	{
		$state='WARNING';
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
		$message="(> $warning%)";
	}
    print "$state : session used $used% $message. Session active: $active / $max|sess=$used;active=$active;max=$max";

    #|session=%.0f",$state,$used,$message,$used);

}
elsif($type eq 'inventory')
{
	my @array;
	my $model = ESIA::SNMP::get($snmp,"$oid_paloalto_chassis");
	my $firmware = ESIA::SNMP::get($snmp,"$oid_paloalto_firmware");
	my $serial = ESIA::SNMP::get($snmp,"$oid_paloalto_serial");
	my $hwver = ESIA::SNMP::get($snmp,"$oid_paloalto_hw_version");
	my %hash = ('name' => 'Palo Alto firewall '.$model,'manufacturer'=>'Palo Alto Networks', 'model' => $model, 'type'=>'CHASSIS','serial'=>$serial,'hardwareVersion' =>$hwver, 'firmwareVersion' => $firmware);
	push(@array, \%hash);
	my $output;
	if($json)
    {
       $output = encode_json(\@array);
    }
    else
    {
		foreach my $line(@array)
        {
			my $hash = \%$line;
			$output.= "name: ".$hash->{name}.", manufacturer: ".$hash->{manufacturer}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
			$output.=", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
		}
    }
    print $output;
}
exit($return);
