#!/usr/bin/perl -w
############################## check_snmp_qsan.pl ########
my $Version='1.0';
# Date : Sep 1 2020
# Author  : Biersart Nicolas
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use JSON;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;

my $PROGNAME='check_snmp_qsan.pl';
my $PROGDESC='Test les système qsan';
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;
my $return=0;
my ($opt_h,$opt_V,$json);
my $warning = 80;
my $critical = 90;
my $type = "raid";
## OIDs ##
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";
my $oid_raid_name   = ".1.3.6.1.4.1.22274.1.2.2.1.1";
my $oid_raid_state  = ".1.3.6.1.4.1.22274.1.2.2.1.6";
my $oid_raid_health = ".1.3.6.1.4.1.22274.1.2.2.1.7";
my $oid_raid_level  = ".1.3.6.1.4.1.22274.1.2.2.1.8";
my $oid_raid_cntrl  = ".1.3.6.1.4.1.22274.1.2.2.1.9";
my $oid_cntrl = ".1.3.6.1.4.1.22274.1.1.1.7";
my $oid_disk_enclosure  =".1.3.6.1.4.1.22274.1.2.1.1.1";
my $oid_disk_status     =".1.3.6.1.4.1.22274.1.2.1.1.4";
my $oid_disk_health     =".1.3.6.1.4.1.22274.1.2.1.1.5";
my $oid_disk_vendor     =".1.3.6.1.4.1.22274.1.2.1.1.7";
my $oid_disk_serial     =".1.3.6.1.4.1.22274.1.2.1.1.8";
my $oid_disk_size		=".1.3.6.1.4.1.22274.1.2.1.1.2";

my $oid_vdisk_name   =".1.3.6.1.4.1.22274.1.2.3.1.1";
my $oid_vdisk_size   =".1.3.6.1.4.1.22274.1.2.3.1.2";
my $oid_vdisk_state  =".1.3.6.1.4.1.22274.1.2.3.1.6";
my $oid_vdisk_health =".1.3.6.1.4.1.22274.1.2.3.1.7";

my $oid_smart_enclosure =".1.3.6.1.4.1.22274.1.3.4.1.1";
my $oid_smart_error     =".1.3.6.1.4.1.22274.1.3.4.1.3";
my $oid_smart_temp      =".1.3.6.1.4.1.22274.1.3.4.1.4";

my $oid_env_local =".1.3.6.1.4.1.22274.1.3.2.1.1";
my $oid_env_type  =".1.3.6.1.4.1.22274.1.3.2.1.2";
my $oid_env_name  =".1.3.6.1.4.1.22274.1.3.2.1.3";
my $oid_env_status =".1.3.6.1.4.1.22274.1.3.2.1.5";

my $oid_inv_firmware = "1.3.6.1.4.1.22274.1.4.1.3";
my $oid_inv_sn  = "1.3.6.1.4.1.22274.1.4.1.4";
my $oid_inv_cpu = "1.3.6.1.4.1.22274.1.4.1.1";
sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
#
#MAIN
#

Getopt::Long::Configure( "bundling" );
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"T=s" => \$type,"type=s" => \$type,
	"J" => \$json,	 "json" => \$json,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
);
# -h & --help print help
# -h & --help print help
if ($opt_h || !defined($snmp))
{
    #affiche l'aide par défaut
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-T stand');
	my @arr;
	push(@arr,["-T","--type","type of test [raid,disk,smart,env,virtual-disk] (default: raid)" ]);
	push(@arr,["-w","--warning","warning threshold in percent  (default: 80)" ]);
	push(@arr,["-c","--critical","critical threshold in percent (default: 90)"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}

my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if($Sysoid_value==-1)
{
	print ("UNKNOWN: Host not SNMP response $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
if($type eq 'raid')
{
    my $status = ESIA::SNMP::get($snmp,"$oid_cntrl");
    if($status eq "0")
    {
        print "OK : Dual controller is OK\n";
    }
    else
    {
        print "CRITICAL : there is a problem on controller\n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
    }
    my $result = $snmp->get_table(Baseoid => $oid_raid_name);
    foreach my $key ( keys %$result)
    {
        #on récupère l'index
        if($key=~ /([0-9]*)$/ )
        {
            my $state = 'OK';
            my $index =$1;
            my $descr =$$result{"$key"};
            my $status = ESIA::SNMP::get($snmp,"$oid_raid_state.$index");
            my $health = ESIA::SNMP::get($snmp,"$oid_raid_health.$index");
            my $lvl = ESIA::SNMP::get($snmp,"$oid_raid_level.$index");
            my $cntrl = ESIA::SNMP::get($snmp,"$oid_raid_cntrl.$index");
            if($status ne 'Online')
            {
                $state = 'CRITICAL';
            }
            if($health ne 'Good')
            {
                $state = 'CRITICAL';
            }
            print "$state : raid $descr ($lvl) is $status and $health on $cntrl\n";
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
        }
    }
}
elsif($type eq 'disk')
{
   	my $result = $snmp->get_table(Baseoid => $oid_disk_enclosure);
    foreach my $key ( keys %$result)
    {
        #on récupère l'index
        if($key=~ /([0-9]*)$/ )
        {
            my $state = 'OK';
            my $index =$1;
            my $descr =$$result{"$key"};
            my $status = ESIA::SNMP::get($snmp,"$oid_disk_status.$index");
            my $health = ESIA::SNMP::get($snmp,"$oid_disk_health.$index");
            my $vendor = ESIA::SNMP::get($snmp,"$oid_disk_vendor.$index");
            my $serial = ESIA::SNMP::get($snmp,"$oid_disk_serial.$index");
            if($status ne 'Online')
            {
                $state = 'CRITICAL';
            }
            if($health ne 'Good')
            {
                $state = 'CRITICAL';
            }
            print "$state : disk in $descr ($vendor: $serial) is $status and $health\n";
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
        }
    }
}
elsif($type eq 'smart')
{

    my $result = $snmp->get_table(Baseoid => $oid_smart_enclosure);
    my $state = 'OK';
    my $output="";
    foreach my $key ( keys %$result)
    {
        #on récupère l'index
        if($key=~ /([0-9]*)$/ )
        {
            my $index =$1;
            my $descr =$$result{"$key"};
            my $smart = ESIA::SNMP::get($snmp,"$oid_smart_error.$index");
            if($smart ne 'N/A')
            {
                $state = 'WARNING';
                $output .= "smart disk error on $descr\n";

            }
        }
    }
    if($state eq 'OK')
    {
        print "OK : All smart test is OK\n";
    }
    else
    {
        print "WARNING: $output";
    }
    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
}
elsif($type eq 'virtual-disk')
{

    my $result = $snmp->get_table(Baseoid => $oid_vdisk_name);
    foreach my $key ( keys %$result)
    {
        #on récupère l'index
        if($key=~ /([0-9]*)$/ )
        {
            my $state = 'OK';
            my $index =$1;
            my $descr =$$result{"$key"};
            my $status = ESIA::SNMP::get($snmp,"$oid_vdisk_state.$index");
            my $health = ESIA::SNMP::get($snmp,"$oid_vdisk_health.$index");
            my $size = ESIA::SNMP::get($snmp,"$oid_vdisk_size.$index");
            if($status ne 'Online')
            {
                $state = 'CRITICAL';
            }
            if($health ne 'Optimal')
            {
                $state = 'CRITICAL';
            }
            print "$state : virtual-disk $descr ($size GB) is $status and $health\n";
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
        }
    }
}
elsif($type eq 'env')
{
    my $total = 0;
    my $error = 0;
    my $output="";
    my $result = $snmp->get_table(Baseoid => $oid_env_status);
    foreach my $key ( keys %$result)
    {
        #on récupère l'index
        if($key=~ /([0-9]*)$/ )
        {
            my $state = 'OK';
            my $index =$1;
            my $status =$$result{"$key"};
            if($status ne 'OK')
            {
                $state = 'CRITICAL';
                $error++;
                my $local = ESIA::SNMP::get($snmp,"$oid_env_local.$index");
                my $type = ESIA::SNMP::get($snmp,"$oid_env_type.$index");
                my $name = ESIA::SNMP::get($snmp,"$oid_env_name.$index");
                $output .= "sensors $name (type: $type, location: $local) is $status\n";
            }
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
        }
        $total++;
    }
    if($return == 0)
    {
        print "OK: All environnement variable are OK (errors: $error, total: $total)\n";
    }
    else
    {
        print "CRITICAL: There are environnement error (errors: $error, total: $total):\n $output";
    }
}
elsif($type eq 'inventory')
{
	my @output;
	my $oid_inv_firmware = "1.3.6.1.4.1.22274.1.4.1.3";
	my $oid_inv_sn  = "1.3.6.1.4.1.22274.1.4.1.4";
	my $oid_inv_cpu = "1.3.6.1.4.1.22274.1.4.1.1";
	my $local = ESIA::SNMP::get($snmp,"$oid_inv_firmware");
	if($local=~ /([0-9A-Za-z]*)\s(.*)/ )
	{
		my $sn = ESIA::SNMP::get($snmp,"$oid_inv_sn");
		my %hash = (
			'name' => "$1",
			'description'=>'',
			'manufacturer'=>'QSAN',
			'model' => $1,
			'type'=>'CHASSIS',
			'serial' => $sn,
			'hardwareVersion' => '',
			'firmwareVersion' => $2,
		);
		push(@output, \%hash);
	}
   	my $result = $snmp->get_table(Baseoid => $oid_disk_enclosure);
    foreach my $key ( keys %$result)
    {
        #on récupère l'index
        if($key=~ /([0-9]*)$/ )
        {
            my $state = 'OK';
            my $index =$1;
            my $descr =$$result{"$key"};
            my $vendor = ESIA::SNMP::get($snmp,"$oid_disk_vendor.$index");
            my $serial = ESIA::SNMP::get($snmp,"$oid_disk_serial.$index");
            my $size =  ESIA::SNMP::get($snmp,"$oid_disk_size.$index");
			my %hash = (
				'name' => "DISK $vendor $size GB",
				'description'=>"enclosure $descr",
				'manufacturer'=>$vendor,
				'model' => $1,
				'type'=>'DISK',
				'serial' => $serial,
				'hardwareVersion' => '',
				'firmwareVersion' => '');
			push(@output, \%hash);
        }
    }
    my $cpuInfo = ESIA::SNMP::get($snmp,"$oid_inv_cpu");
    my $vendor = 'unknown';
    if($cpuInfo =~ /intel/i )
    {
    	 $vendor ='Intel';
    }
   	elsif($cpuInfo =~ /amd/i )
   	{
   		$vendor ='AMD';
   	}
    my %hash = ('name' => $cpuInfo,
				'description'=>"",
				'manufacturer'=>$vendor,
				'model' => $cpuInfo,
				'type'=>'CPU',
				'serial' => '',
				'hardwareVersion' => '',
				'firmwareVersion' => '',
			);
	push(@output, \%hash);
	if($json)
	{
		my $out = encode_json(\@output);
		print $out;
	}
	else
    {
        my $size = @output;
        if($size != 0)
        {
            foreach my $line ( @output)
            {

                my $hash = \%$line;
                print "name: ".$hash->{name}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
                print ", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
            }
            exit 0;
        }
	}
}
else
{
    print "UNKNOWN: type request not found. Check parameter\n";
    exit $ERRORS{"UNKNOWN"};
}

exit($return);
