#!/usr/bin/perl
#
use strict;
use warnings;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use vars qw($PROGNAME);
use Net::SNMP;
use Getopt::Long;
use ESIA::SNMP;
use ESIA::PLUGIN;
use JSON;
my $PROGNAME = "check_snmp_ratiran.pl";
my $PROGDESC = "Check pdu Raritan by SNMP";
my $PROGVERSION = "1.0";

my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

ESIA::PLUGIN::arm_end_signal($TIMEOUT);
Getopt::Long::Configure( "bundling" );
my ($opt_h,$opt_V);
my $warning=80;
my $critical=90;


my $ratiran_descr   			 = "1.3.6.1.2.1.1.1.0";
my $ratiran_device_model_name 	 = "1.3.6.1.4.1.13742.6.3.2.1.1.3.1";
my $ratiran_device_serial_number = "1.3.6.1.4.1.13742.6.3.2.1.1.4.1";
my $ratiran_device_name			 = "1.3.6.1.4.1.13742.6.3.2.2.1.13.1";
my $ratiran_software_version     = "1.3.6.1.4.1.13742.6.3.2.3.1.6.1.1.1";
my $ratiran_inlet_rated_voltage  = "1.3.6.1.4.1.13742.6.3.3.3.1.6";
my $ratiran_inlet_rated_current  = "1.3.6.1.4.1.13742.6.3.3.3.1.7.1.1";
my $ratiran_outlet_rated_voltage = "1.3.6.1.4.1.13742.6.3.5.3.1.6";
my $ratiran_outlet_rated_current = "1.3.6.1.4.1.13742.6.3.5.3.1.7";
my $ratiran_outlet_name          = "1.3.6.1.4.1.13742.6.3.5.3.1.3.1";
my $raritan_outlet_state 		 = "1.3.6.1.4.1.13742.6.4.1.2.1.3.1";
my $raritan_outlet_power_out	 = "1.3.6.1.4.1.13742.6.4.1.2.1.2.1"; # Power cycle an outlet   SI == 1 => ON
my %raritan_outlet_state_text = ( "-1"  => "unavailable",
                                    "0"  => "open",
                                    "1" => "closed",
                                    "2"  => "belowLowerCritical",
                                    "3"  => "belowLowerWarning",
                                    "4"  => "normal",
                                    "5"  => "aboveUpperWarning",
                                    "6"  => "aboveUpperCritical",
                                    "7"  => "on",
                                    "8"  => "off",
                                    "9"  => "detected",
                                    "10"  => "notDetected",
                                    "11"  => "alarmed",
                                    "12"  => "ok",
                                    "13"  => "unknown",
                                    "14"  => "fail",
                                    "15"  => "yes",
                                    "16"  => "no",
                                    "17"  => "standby",
                                    "18"  => "one",
                                    "19"  => "two",
                                    "20"  => "inSync",
                                    "21"  => "outOfSync",
                                    "22"  => "i1OpenFault",
                                    "23"  => "i1ShortFault",
                                    "24"  => "i2OpenFault",
                                    "25"  => "i2ShortFault",
                                    "26"  => "fault",
                                    "27"  => "warning",
                                    "28"  => "critical",
                                    "29"  => "selfTest",
                                    "30"  => "nonRedundant");
my %raritan_outlet_state_text_state = ( "-1"  => "UNKNOWN",
                                    "0"  => "OK",
                                    "1" => "CRITICAL",
                                    "2"  => "CRITICAL",
                                    "3"  => "WARNING",
                                    "4"  => "OK",
                                    "5"  => "WARNING",
                                    "6"  => "CRITICAL",
                                    "7"  => "OK",
                                    "8"  => "CRITICAL",
                                    "9"  => "OK",
                                    "10"  => "WARNING",
                                    "11"  => "WARNING",
                                    "12"  => "OK",
                                    "13"  => "UNKNOWN",
                                    "14"  => "CRITICAL",
                                    "15"  => "OK",
                                    "16"  => "CRITICAL",
                                    "17"  => "WARNING",
                                    "18"  => "UNKNOWN",
                                    "19"  => "twUNKNOWNo",
                                    "20"  => "WARNING",
                                    "21"  => "CRITICAL",
                                    "22"  => "CRITICAL",
                                    "23"  => "CRITICAL",
                                    "24"  => "CRITICAL",
                                    "25"  => "CRITICAL",
                                    "26"  => "CRITICAL",
                                    "27"  => "WARNING",
                                    "28"  => "CRITICAL",
                                    "29"  => "OK",
                                    "30"  => "UNKNOWN");
my $ratiran_pdu_current_load 	 = "1.3.6.1.4.1.13742.6.5.2.3.1.4.1.1.1"; # Gauge32: 10 (A)  => 8,686
my $ratiran_pdu_input_voltage 	 = "1.3.6.1.4.1.13742.6.5.2.3.1.4.1.1.4"; # Gauge32: 119 (V) => 229
my $ratiran_pdu_input_realpower	 = "1.3.6.1.4.1.13742.6.5.2.3.1.4.1.1.5"; # Gauge32: 10 (W) => 1806
my $ratiran_pdu_input_power 	 = "1.3.6.1.4.1.13742.6.5.2.3.1.4.1.1.6"; # Gauge32: 122 (voltamp) => 1985
#my $ratiran_pdu_input_powerfactor= "1.3.6.1.4.1.13742.6.5.2.3.1.4.1.1.7"; # Gauge32: 8 (none) => 91
my $ratiran_pdu_activeEnergy     = "1.3.6.1.4.1.13742.6.5.2.3.1.4.1.1.8"; # Gauge32: 193 359 (wattHour) => 21844,973

#inletSensorDecimalDigits
my $inletSensorDecimalDigits_load 		= "1.3.6.1.4.1.13742.6.3.3.4.1.7.1.1.1";
my $inletSensorDecimalDigits_voltage 	= "1.3.6.1.4.1.13742.6.3.3.4.1.7.1.1.4";
my $inletSensorDecimalDigits_realpower   = "1.3.6.1.4.1.13742.6.3.3.4.1.7.1.1.5";
my $inletSensorDecimalDigits_power 		= "1.3.6.1.4.1.13742.6.3.3.4.1.7.1.1.6";
#my $inletSensorDecimalDigits_powerfactor = "1.3.6.1.4.1.13742.6.3.3.4.1.7.1.1.7";
my $inletSensorDecimalDigits_activeEnergy= "1.3.6.1.4.1.13742.6.3.3.4.1.7.1.1.8";


my $json;
my $type='env';
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
		"V"   => \$opt_V,	 "version"    => \$opt_V,
		"h"   => \$opt_h,	 "help"       => \$opt_h,
        "T=s" => \$type,	 "type=s" => \$type,
        "j" => \$json,	 "json" => \$json,
		"w=s" => \$warning,	 "warning=s" => \$warning,
		"c=s" => \$critical, "critical=s" => \$critical,
	  );


# -h & --help print help
if ($opt_h || !defined($snmp))
{
    #affiche l'aide par défaut
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-m namestorage');
	my @arr;
	push(@arr,["-T","--type","type of test [env|outlet|inventory] default: env" ]);
	push(@arr,["-j","--json","json output for inventory" ]);
	push(@arr,["-w","--warning","warning threshold in percent" ]);
	push(@arr,["-c","--critical","critical threshold in percent"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
# -V & --version print version
if ($opt_V)
{
	print "$PROGNAME version $PROGVERSION\n";
	exit $ERRORS{'OK'};
}

my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if($Sysoid_value==-1)
{
	print "UNKNOWN: Host not SNMP response $snmperror\n";
	exit $ERRORS{'UNKNOWN'};
}
my $return = 0;
if($type eq 'env')
{
    my $model_name = ESIA::SNMP::get($snmp,"$ratiran_device_model_name");
    my $serial_number = ESIA::SNMP::get($snmp,"$ratiran_device_serial_number");
    my $device_name = ESIA::SNMP::get($snmp,"$ratiran_device_name");
    my $software_version = ESIA::SNMP::get($snmp,"$ratiran_software_version");
    print "Name: $device_name Model: $model_name, serial number: $serial_number, version: $software_version \n";



    my $current_load = ESIA::SNMP::get($snmp,"$ratiran_pdu_current_load");
    my $digit_load  = ESIA::SNMP::get($snmp,"$inletSensorDecimalDigits_load");
    $current_load=sprintf("%.2f",$current_load/(10**$digit_load));	 
    
    my $voltage = ESIA::SNMP::get($snmp,"$ratiran_pdu_input_voltage");
    my $digit_voltage  = ESIA::SNMP::get($snmp,"$inletSensorDecimalDigits_voltage");
    $voltage = $voltage/(10**$digit_voltage);

    my $power = ESIA::SNMP::get($snmp,"$ratiran_pdu_input_power");
    my $digit_power  = ESIA::SNMP::get($snmp,"$inletSensorDecimalDigits_power");
    $power = $power/(10**$digit_power);
	if($current_load > 0)
	{
        my $max_amp = ESIA::SNMP::get($snmp,"$ratiran_inlet_rated_current");
	    my $max = ($max_amp =~ /(\d+)A/);
	    $max=$1;
	    my $used = sprintf("%.2f",(($current_load/$max)*100));
        my $state = 'OK';
        my $message =" < $warning %";
	    if($used > $critical)
	    {
            $state = 'CRITICAL';
            $message =" > $critical %";
	    }
	    elsif($used > $warning)
	    {
            $state = 'WARNING';
            $message =" > $warning %";
	    }	
        print "$state: PDU power is $power W \n";
        print "$state: PDU voltage is $voltage V \n";
        print "$state: PDU load is $used % ($current_load A)$message|percent=0,amp=$current_load,vol=$voltage,pow=$power,warning=$warning,critical=$critical \n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state})
    }
    else
    {
        print "OK: PDU power is $power W \n";
        print "OK: PDU voltage is $voltage V \n";
        print "CRITICAL: PDU load is $current_load A. Device is down !|percent=0,amp=$current_load,vol=$voltage,pow=$power,warning=$warning,critical=$critical \n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
    }
}
elsif($type eq 'outlet')
{
    my $result = $snmp->get_table(Baseoid => $raritan_outlet_state);
	if (!defined($result))
	{
	   	print "UNKNOWN: The status of an outlet : na\n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
	}
    else
    {
        foreach my $key (sort keys %$result)
        {								    
            if($key =~/3.6.1.4.1.13742.6.4.1.2.1.3.1\.([0-9]*)/)
            {
				my $id = $1;
                my $stateout = $$result{"$key"};
                my $showdesc=$id;
                my $name = ESIA::SNMP::get($snmp,"$ratiran_outlet_name.$id");
                if($name ne '')
                {
                    $showdesc="$id ($name)";
                } 
                my $state=$raritan_outlet_state_text_state{$stateout};
                my $power_out  = ESIA::SNMP::get($snmp,"$raritan_outlet_power_out.$id");
	 			if($power_out==1)
	 			{
	 				$power_out="on";
				}
				else
	 			{
	 				$power_out="off";
                    if( $state ne 'CRITICAL')
                    {
                        $state='WARNING';
                    }
	 			}
            	print "$state: Outlet $showdesc : power : $power_out ; state: ".$raritan_outlet_state_text{$stateout}."\n";
	 		    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state})
            }
        }
    }
}
elsif($type eq 'inventory')
{
	my @array;

    my $model = ESIA::SNMP::get($snmp,"$ratiran_device_model_name");
    my $serial = ESIA::SNMP::get($snmp,"$ratiran_device_serial_number");
    my $firmware = ESIA::SNMP::get($snmp,"$ratiran_software_version");

	my %hash = ('name' => 'Raritan PDU','manufacturer'=>'Raritan', 'model' => $model, 'type'=>'CHASSIS','serial'=>$serial,'hardwareVersion' =>'', 'firmwareVersion' => $firmware);
	push(@array, \%hash);
	my $output;
	if($json)
    {
       $output = encode_json(\@array);
    }
    else
    {
		foreach my $line(@array)
        {
			my $hash = \%$line;
			$output.= "name: ".$hash->{name}.", manufacturer: ".$hash->{manufacturer}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
			$output.=", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
		}
    }
    print $output;
}
exit $return;
