#!/usr/bin/perl -w
############################## check_snmp_hp5400_temp.pl ########
my $Version='1.0';
# Date : 19 février 2019
# Author  : Nicolas Biersart
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;
my $PROGNAME='check_snmp_ubiquity_switch.pl';
my $PROGDESC = "Check Ubiquity switch CPU, ram and environnement sensors";
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;


my $return=0;
my ($opt_h,$opt_V);
my $warning='';
my $critical='';
my $type = 'env';
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";
ESIA::PLUGIN::arm_end_signal($TIMEOUT);

#CPU
my $oid_ubiquity_cpu_no_parser 	= "1.3.6.1.4.1.4413.1.1.1.1.4.13.0"; #
my $oid_ubiquity_cpu_parser 	= "1.3.6.1.4.1.4413.1.1.1.1.4.9.0"; #
# MEM
my $oid_ubiquity_mem_total	    = "1.3.6.1.4.1.4413.1.1.1.1.4.2.0"; # Total KBytes
my $oid_ubiquity_mem_free		= "1.3.6.1.4.1.4413.1.1.1.1.4.1.0"; # Free KBytes

# FAN
my $oid_ubiquity_fan_state	    = "1.3.6.1.4.1.4413.1.1.43.1.6.1.3";
my %oid_ubiquity_fan_state_text = ( "0"  => "Unknown","1" => "notpresent","2"  => "operational","3"  => "failed","4"  => "powering","5"  => "nopower","6"  => "notpowering","7"  => "incompatible");
my %oid_ubiquity_fan_status = ( "0"  => "UNKNOWN","1"  => "UNKNOWN","2" => "OK","3"  => "CRITICAL","4"  => "WARNING","5"  => "CRITICAL","6"  => "WARNING","7"  => "WARNING");
#Power Supply
my $oid_ubiquity_power_state	= "1.3.6.1.4.1.4413.1.1.43.1.7.1.3";

#Temp
my $oid_ubiquity_temperature		= "1.3.6.1.4.1.4413.1.1.43.1.8.1.5";
my $oid_ubiquity_temperature_state	= "1.3.6.1.4.1.4413.1.1.43.1.8.1.4";
my %oid_ubiquity_temperature_state_text = ( "0"  => "low","1" => "normal","2"  => "warning","3"  => "critical","4"  => "shutdown","5"  => "notpresent","6"  => "notoperational");
my %oid_ubiquity_temperature_status = ( "0"  => "WARNING","1"  => "OK","2" => "WARNING","3"  => "CRITICAL","4"  => "WARNING","5"  => "OK","6"  => "WARNING");
#
#MAIN
#
&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);

GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
    "T=s" => \$type,"type=s" => \$type,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
);
# -h & --help print help
if ($opt_h)
{
#affiche l'aide par défaut
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-T state');
	my @arr;
	push(@arr,["-T","--type","type of test [cpu,mem,env] (default: cpu)" ]);
	push(@arr,["-w","--warning","warning threshold in percent" ]);
	push(@arr,["-c","--critical","critical threshold in percent"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
#on interroge en SNMP pour récupérer les informations
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if($Sysoid_value==-1)
{
	print ("UNKNOWN: Host not SNMP response $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
# test des paramètre par défaut
if($warning eq '' )
{
    if($type eq 'cpu' || $type eq 'mem')
    {
        $warning=80;
    }
    else
    {
    	$warning=55;
    }
}
if($critical eq '')
{
    if($type eq 'cpu'|| $type eq 'mem')
    {
        $critical=80;
    }
    else
    {
    	$critical=65;
    }
}
#
#Ubiquity switch
#

if($type eq 'cpu')
{
	my $state = 'OK';
	my $threshold = ESIA::SNMP::get($snmp,"$oid_ubiquity_cpu_no_parser");
	if(!$threshold =~/[0-9]*/)
	{
		my $parser = ESIA::SNMP::get($snmp,"$oid_ubiquity_cpu_parser");
		if($parser =~/300\s+Secs\s+\(([\s0-9\.]*)[\%\s]*\)/)
		{
			$threshold=$1;
			$threshold =~ s/^\s+|\s+$//g;
		}
		else
		{
			$state='UNKNOWN';
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
			print "UNKNOWN: bad CPU value: $parser\n";
		}
	}
    my $message="(< $warning)";
    if($threshold >= $critical)
    {
    	$state='CRITICAL';
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
		$message="(> $critical)";
	}
	elsif($threshold >= $warning)
	{
		$state='WARNING';
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
		$message="(> $warning)";
	}
	printf("%s : CPU used %.1f%% %s|cpu=%.0f",$state,$threshold,$message,$threshold);
}
elsif($type eq 'mem')
{
	my $state = 'OK';
	my $total = ESIA::SNMP::get($snmp,"$oid_ubiquity_mem_total");
	my $memory = ESIA::SNMP::get($snmp,"$oid_ubiquity_mem_free");
 	my $used = 100 - ($memory / $total)*100;
	my $message="(< $warning)";
	if($used >= $critical)
	{
		$state='CRITICAL';
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
		$message="(> $critical)";
	}
	elsif($used >= $warning)
	{
		$state='WARNING';
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
		$message="(> $warning)";
	}
    printf("%s :memory used %.0f%% %s|mem=%.0f",$state,$used,$message,$used);
}
if($type eq 'env')
{
	my $result = $snmp->get_table(Baseoid => $oid_ubiquity_fan_state);
	foreach my $key ( keys %$result)
	{
		#on récupère l'index
	   	if($key=~ /([0-9]*)$/ )
	   	{
                my $index =$1;
		        my $status =$$result{"$key"};
                my $state = $oid_ubiquity_fan_status{$status};
                print "$state : Fan $index is ".$oid_ubiquity_fan_state_text{$status}."\n";
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
		}

	}
	$result = $snmp->get_table(Baseoid => $oid_ubiquity_power_state);
	foreach my $key ( keys %$result)
	{
		#on récupère l'index
	   	if($key=~ /([0-9]*)$/ )
	   	{
                my $index =$1;
		        my $status =$$result{"$key"};
                my $state = $oid_ubiquity_fan_status{$status};
                print "$state : Power supply $index is ".$oid_ubiquity_fan_state_text{$status}."\n";
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
		}

	}
	$result = $snmp->get_table(Baseoid => $oid_ubiquity_temperature);
	my $total=0;
	my $i=0;
	foreach my $key ( sort keys %$result)
	{
		#on récupère l'index
	   	if($key=~ /([0-9]*\.[0-9]*)$/ )
	   	{
                my $index =$1;
		        my $temperature =$$result{"$key"};
		        my $status = ESIA::SNMP::get($snmp,"$oid_ubiquity_temperature_state.$index");
                my $state = $oid_ubiquity_temperature_status{$status};
                print "$state : Sensor $index is ".$oid_ubiquity_temperature_state_text{$status}.", measured temperature: $temperature°C\n";
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
                $total=$total+$temperature;
                $i++;
		}
	}
	my $average = sprintf('%.1f',$total/$i);
	my $state = 'OK';
	my $message="(< $warning)";
	if($average >= $critical)
	{
		$state='CRITICAL';
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
		$message="(> $critical)";
	}
	elsif($average >= $warning)
	{
		$state='WARNING';
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
		$message="(> $warning)";
	}
	print "$state: average temperature: $average°C $message|temp=$average\n";
}
exit($return);
