#!/usr/bin/php
<?php
$DEFAULT_TIMEOUT = 50;
$TIME_SLEEP = 0.5;
$STR_STATUS=array("OK","WARNING","CRITICAL","UNKNOWN");
$INT_STATUS=array_flip($STR_STATUS);
$PRIORITIES_STATES=array("CRITICAL","WARNING","UNKNOWN","OK");

//Récupération et vérifications des paramètres.
$type_list = array('load','mem','storage','inodes','uptime','time','interface');

$params = new Parameters();
$params->addOpt('','h','help',NULL,'print this help');
$params->addOpt(':','u','uri','<url>','Powershell daemon url (http://127.0.0.1:2082 by default)');
$params->addOpt(':','H','host','<host>','Host IP ou FQDN');
$params->addOpt(':','U','user','<login>','Login or username');
$params->addOpt(':','P','password','<password/pathfile>','password or path for pubkey, sharedkey or special authentication options');
$params->addOpt(':','A','authfile','<filepath>','authentication file filepath form set default user, password parameters');
$params->addOpt(':','T','type','<test_type>','To specifie test type');
$params->addOpt(':','t','timeout','<second>','Timeout for script usage',$DEFAULT_TIMEOUT);
$params->addOpt(':','m','match','<regex>','To match specific value');
$params->addOpt(':','s','speed','<bps>','Must be auto or BP speed in bps');
$params->addOpt(':','w','warning','<treshold>','warning treshold');
$params->addOpt(':','c','critical','<treshold>','critical treshold');

$params->addUsage('-H <host> -U <user> -P <password -T <type> -w <treshold> -c <treshold>');
$params->addUsage('-H <host> [-U <user>] -A <authfile> -T <type> -w <treshold> -c <treshold>');

if( $params->getParam('help') )
{
    $params->print_help();
    exit($INT_STATUS['UNKNOWN']);
}

foreach(array('type','warning','critical') as $key)
{
    $param = $params->getParam($key);
    $params_opt_str = $params->getParamOptionsString($key);
    if( empty($param) )
    {
        echo "Missing required option $params_opt_str\n\n";
        $params->print_usage();
        exit($INT_STATUS['UNKNOWN']);
    }
}

$o_type = $params->getParam('type');
if(!in_array($o_type,$type_list))
{
    $params_opt_str = $params->getParamOptionsString('type');
    $str = "Incorrect value $params_opt_str=$o_type.";
    $str.= " This value must be : ".implode(", ",$type_list);
    exit_state('UNKNOWN',$str);
}

$o_authfile_path = $params->getParam('authfile');
if(!empty($o_authfile_path))
{
    $params_opt_str = $params->getParamOptionsString('authfile');
//	print_r($params_opt_str);
    if( !file_exists($o_authfile_path) )
    {
        exit_state('UNKNOWN',"Unable to found $params_opt_str=$o_authfile_path");
    }
    $auth_file_data = file_get_contents($o_authfile_path);
    if( empty($auth_file_data))
    {
        exit_state('UNKNOWN',"Unable to open $params_opt_str=$o_authfile_path");
    }
    $auth_file_data = json_decode($auth_file_data,TRUE);
    if( !is_array($auth_file_data))
    {
        exit_state('UNKNOWN',"Unable to parse $params_opt_str=$o_authfile_path");
    }
}

$auth_params = array();
foreach(array('host','user','password') as $key)
{
    $auth_params[$key] = $params->getParam($key,@$auth_file_data[$key]);
    $params_opt_str = $params->getParamOptionsString($key);
    if( empty($auth_params[$key]))
    {
        exit_state('UNKNOWN',"Missing required option $params_opt_str or authfile parameter $key");
    }
}

//Au cas ou un $IP serait passé pour le confile en plus du host
if( strpos('$IP',$auth_params['host']) !== FALSE && preg_match('/(\d{1,3}\.\d{1,3}.\d{1,3}\.\d{1,3})/',basename($o_authfile_path),$matches))
{
        if (filter_var($matches[1], FILTER_VALIDATE_IP))
        {
                $auth_params['host'] = str_replace('$IP', $matches[1], $auth_params['host']);
        }
}

//Armement timeout
declare(ticks = 1);
function sig_handler($signo)
{
        global $timeout;
        exit_state("UNKNOWN","Timeout after $timeout seconds");
}

$timeout = $params->getParam('timeout');
pcntl_alarm($timeout);
pcntl_signal(SIGALRM, "sig_handler");

// Actions en fonction du type
$user_auth_list[] = array($auth_params['user'],$auth_params['password']);
//$user_auth_list[] = array('root',$auth_params['password']);
//$user_auth_list[] = array('postgres','');

//$output = ssh_postgres_request($auth_params['host'],$user_auth_list,$dbname,$sql,0.01);


$first = array_shift($user_auth_list);
$host_user = $first[0];
$host_pass = $first[1];

//print_r($user_auth_list);
try
{
    $shell = ssh_shell_open($auth_params['host'],$host_user,$host_pass);
}
catch(Exception $ex)
{
    exit_state('UNKNOWN','connet'.$ex->getMessage());
}
//ssh_shell_multi_su($shell,array(array('root',$host_pass)),$TIME_SLEEP);
ssh_shell_clear($shell);

// CPU
$warning = $params->getParam('warning');
$critical = $params->getParam('critical');
$type = $params->getParam('type');
if( $type == 'load' )
{
    validate_treshold_calcul_max($warning,$critical,100);
    $ncores = trim(ssh_shell_oneline_cmd($shell,"grep 'model name' /proc/cpuinfo|wc -l",1,$TIME_SLEEP));
    $uptime = ssh_shell_oneline_cmd($shell,"uptime",1,$TIME_SLEEP);
    ssh_shell_close($shell);
    $uptime = str_replace(',','.',$uptime);
    if($ncores == 0 || !preg_match("/load average: (\d+\.\d+)\. (\d+\.\d+)\. (\d+\.\d+)/",$uptime,$matches))
    {
        exit_state('UNKNOWN','Unable to get CPU core or load average informations');
    }
    $load_average_1m = $matches[1];
    $load_average_5m = $matches[2];
    $load_average_15m = $matches[3];

    if( $load_average_5m < 0.01)
    {
        $cpu_usage = 0;
    }
    else
    {
        $cpu_usage = ($load_average_5m/$ncores)*100;
        $cpu_usage = round($cpu_usage,2);
    }
    if( $cpu_usage > 100 )
    {
        $cpu_usage = 100;
    }

    $str = "$ncores CPU, average load $cpu_usage%";
    $result = calcul_max_state($warning,$critical,$cpu_usage,'%');

    exit_state($result['state'],"$str ".$result['reason']);
}
// Mem
elseif( $type == 'mem' )
{
    $warning = explode(',',$warning);
    $critical = explode(',',$critical);

    if( count($warning) != 2 || count($critical) != 2 )
    {
        exit_state("UNKNOWN","There are not 2 warnings and 2 critical for the RAM and SWAP.");
    }
    $warn_ram = $warning[0];
    $crit_ram = $critical[0];
    $warn_swp = $warning[1];
    $crit_swp = $critical[1];
    validate_treshold_calcul_max($warn_ram,$crit_ram,100);
    validate_treshold_calcul_max($warn_swp,$crit_swp,100);

    $memory = ssh_shell_oneline_cmd($shell,'free -bw|tr -s " "|cut -d " " -f2-7',3,$TIME_SLEEP);
    ssh_shell_close($shell);

    $memory = explode("\n",$memory);
    $swap = explode(' ',$memory[2]);
    $memory = explode(' ',$memory[1]);

    $mem_usage = intval($memory[1])+intval($memory[3]+intval($memory[4])); //used + shared + buffers (no cache)
    $mem_total = $memory[0];
    $mem_perc = round($mem_usage/$mem_total*100,2);
    $mem_usage_MB = intval($mem_usage/1024/1024);
    $mem_total_MB = intval($mem_total/1024/1024);

    $swap_usage = intval($swap[1]);
    $swap_total = intval($swap[0]);
    if( $swap_total > 0)
    {
        $swap_perc = round($swap_usage/$swap_total*100,2);
    }
    else
    {
        $swap_perc = 'NAN';
    }
    $swap_usage_MB = intval($swap_usage/1024/1024);
    $swap_total_MB = intval($swap_total/1024/1024);

    $global_state = 'OK';
    $result = calcul_max_state($warn_ram,$crit_ram,$mem_perc,'%');
    $str = $result['state']. ": Ram: $mem_perc% (${mem_usage_MB}MB/${mem_total_MB}MB) ".$result['reason']."\n";

    $global_state = calcul_state($global_state,$result['state']);

    $result = calcul_max_state($warn_swp,$crit_swp,$swap_perc,'%');
    $str .= $result['state']. ": Swap: $swap_perc% (${swap_usage_MB}MB/${swap_total_MB}MB) ".$result['reason']."\n";

    $str .= "|ram=$mem_perc;rambyte=$mem_usage;ramTotal=$mem_total;swap=$swap_perc;swapByte=$swap_usage;swapTotal=$swap_total\n";

    $global_state = calcul_state($global_state,$result['state']);
    echo $str;
    exit($global_state);
}
// Disque storage
elseif( $type == 'storage' )
{
    $disk_select = $params->getParam('match');
    validate_treshold_calcul_max($warning,$critical,100);
    $output = ssh_shell_cmd($shell,'df -k|tr -s " "',$TIME_SLEEP);
    ssh_shell_close($shell);
    check_df($output,$disk_select,$warning,$critical,'','MB',1024);
}
// Disque inodes
elseif( $type == 'inodes' )
{
    exit_state('UNKNOWN','This options is not available for Innovaphones');
    $disk_select = $params->getParam('match');
    validate_treshold_calcul_max($warning,$critical,100);
    $output = ssh_shell_cmd($shell,'df -i|tr -s " "',$TIME_SLEEP);
    ssh_shell_close($shell);
    check_df($output,$disk_select,$warning,$critical,'','',1);
}
// Uptime
elseif( $type == "uptime" )
{
    validate_treshold_calcul_min($warning,$critical,0);
    $version = trim(ssh_shell_oneline_cmd($shell,'uname -sr',1,$TIME_SLEEP));
    $hostname = trim(ssh_shell_oneline_cmd($shell,'hostname',1,$TIME_SLEEP));
    $uptime = trim(ssh_shell_oneline_cmd($shell,'cat /proc/uptime',1,$TIME_SLEEP));
    ssh_shell_close($shell);

    $uptime = explode(' ',$uptime)[0];
    $nb = 1;
    $node = str_replace(' '," $hostname ",$version,$nb);
    $period = secondsToPeriod($uptime);

    $str = "$node - up $period";
    $result = calcul_min_state($warning,$critical,$uptime,' seconds');
    exit_state($result['state'],"$str".$result['reason']);
}
//Syncho temps
elseif( $type == "time" )
{
    validate_treshold_calcul_max($warning,$critical,NULL);

    //On récupére X fois l'information et on ne garde que celui qui a le taux d'erreur le plus faible.
    $date_calc = array();
    $date_min_error_idx = 0;
    $start_date = microtime(true);
    for($i=0;$i<3;$i++)
    {
        $read_date = trim(ssh_shell_oneline_cmd($shell,'date +%s.%N',1,$TIME_SLEEP));
        if( preg_match('/(\d+)\.\%N/',$read_date,$matches))
        {
            $read_date = $matches[1];
        }
        $end_date = microtime(true);
        $exec_time = $end_date-$start_date;
        $date_calc[$i]['local_first'] = $start_date;
        $date_calc[$i]['local_end'] = $end_date;
        $date_calc[$i]['local_estimated'] = ($exec_time/2) + $start_date;
        $date_calc[$i]['remote_date'] = $read_date;
        $date_calc[$i]['ratio_error'] = $end_date - $start_date;
        if( $date_calc[$i]['ratio_error'] < $date_calc[$date_min_error_idx]['ratio_error'] )
        {
            $date_min_error_idx = $i;
        }
        $date_calc[$i]['diff'] = $date_calc[$i]['local_estimated'] - $read_date;

        $start_date = $end_date;
    }
    ssh_shell_close($shell);

    $date_calc=$date_calc[$date_min_error_idx];

    if( $date_calc['local_first'] < $date_calc['remote_date'] && $date_calc['remote_date'] < $date_calc['local_end'])
    {
        $result['state'] = 'OK';
        $result['reason'] = ". Remote date between script starting end stopping date";
    }
    else
    {
        $result = calcul_max_state($warning,$critical,$date_calc['diff'],' seconds');
    }

    $str = sprintf("%s time differ by %.2f seconds (error treshold : %.2f)",$auth_params['host'],$date_calc['diff'],$date_calc['ratio_error']);
    exit_state($result['state'],"$str".$result['reason']);
}
// Bandwith
elseif($type == "interface")
{
    validate_treshold_calcul_max($warning,$critical,100);

    $select_iface = $params->getParam('match');
    $param_speed = $params->getParam('speed');
    $uniq_ident =  md5(json_encode($auth_params)."$select_iface");
    $filepath = sys_get_temp_dir() .'/'. basename(__FILE__) . $uniq_ident;

    $loaded = array();
    if( is_file($filepath) )
    {
        $loaded = @file_get_contents($filepath);
        $loaded = json_decode($loaded,TRUE);
        if( $loaded == FALSE )
        {
            $loaded = array();
        }
    }

    $timestamp = time();
    $data = ssh_shell_cmd($shell,'cat /proc/net/dev| tr "|" " " |tr -s " "|cut -d " " -f2,3,5,11,13',$TIME_SLEEP);
    array_shift($data); //Inter-| Receive | Transmit
    $header_col = array_shift($data);
    if($header_col != 'face bytes errs bytes errs')
    {
        exit_state('UNKNOWN','Unable to get interface bandwith informations');
    }

    $parsed = array();
    foreach($data as $row)
    {
        $ret_row = array();
        $cell = explode(' ',$row);
        $iface = rtrim($cell[0],':');
        if( !empty($select_iface) && !preg_match("#$select_iface#",$iface))
        {
            continue;
        }
        $ret_row['time'] = $timestamp;
        $ret_row['rx_bytes'] = $cell[1];
        $ret_row['rx_errs'] = $cell[2];
        $ret_row['tx_bytes'] = $cell[3];
        $ret_row['tx_errs'] = $cell[4];
        $parsed[$iface] = $ret_row;
    }

    if( !@file_put_contents($filepath,json_encode($parsed)))
    {
        exit_state('UNKNOWN',"Unable to save data into tmp file $filepath");
    }

    foreach($parsed as $iface => $row)
    {
        $speed = 'none';
        if( $iface != 'lo' )
        {
            $speed = trim(ssh_shell_oneline_cmd($shell,"cat /sys/class/net/$iface/speed",1,$TIME_SLEEP));
        }
        if( !is_numeric($speed))
        {
            $speed = -1;
        }
        $parsed[$iface]['speed'] = $speed;
    }
    ssh_shell_close($shell);

    $calculated = array();
    foreach($parsed as $iface => $row)
    {
        foreach( array('time','rx_bytes','rx_errs','tx_bytes','tx_errs') as $key)
        {
            if(isset($loaded[$iface][$key]))
            {
                $calculated[$iface][$key] = $parsed[$iface][$key] - $loaded[$iface][$key];
            }
            else
            {
                $calculated[$iface][$key] = 0;
            }
        }
        if( !empty($param_speed) && $param_speed != "auto" )
        {
            $calculated[$iface]['speed'] = $param_speed;
        }
        else
        {
            $calculated[$iface]['speed'] = $parsed[$iface]['speed'];
        }
    }

    $str = '';
    $global_state = 'OK';
    $ifaces_stats = array();
    foreach($calculated as $iface => $row)
    {
        if(  $row['time']  == 0)
        {
            $row['time']  = 1;
        }
        $inBits = $row['rx_bytes']*8 / $row['time'];
        $outBits = $row['tx_bytes']*8 / $row['time'];
        $inErrors = $row['rx_errs'] / $row['time'];
        $outErrors = $row['tx_errs'] / $row['time'];

        $inStr = bitsToHumanStr($inBits);
        $outStr = bitsToHumanStr($outBits);

        if( $row['speed'] == -1 )
        {
            $state = 'UNKNOWN';
            $speed_str = ". No speed detected (Use -s, --speed option)";

            $warningBits = 0;
            $criticalBits = 0;
        }
        else
        {
            $speedBits = $row['speed'] * 1000000;
            $warningBits = ($speedBits/ 100) * $warning;
            $criticalBits = ($speedBits / 100) * $critical;

            $result = calcul_max_state($warningBits,$criticalBits,$inBits,'');
            $state = $result['state'];
            $result = calcul_max_state($warningBits,$criticalBits,$outBits,'');
            $state = calcul_state($state,$result['state']);
            $state = $STR_STATUS[$state];
            $speed_str = "speed=".$row['speed']. "Mb/s";
        }
        $str.= "$state: $iface: in=$inStr out=$outStr $speed_str\n";
        $global_state = calcul_state($state,$global_state);

        $retRow['in'] = $inBits;
        $retRow['out'] = $outBits;
        $retRow['warn'] = $warningBits;
        $retRow['crit'] = $criticalBits;
        $retRow['inErr'] = $inErrors;
        $retRow['outErr'] = $outErrors;
        $ifaces_stats[] = $retRow;
    }

    if( count($ifaces_stats) == 0)
    {
        exit_state('UNKNOWN',"Interface $select_iface not FOUND");
    }
    elseif(count($ifaces_stats) == 1 )
    {
        $iface = array_shift($ifaces_stats);
        $str = trim($str);
        $str.= sprintf("|data:in=%.2fdata:out=%.2f:warning:%d:critical:%d:inErrors:%.2f:outErrors:%.2f",$iface['in'],$iface['out'],$iface['warn'],$iface['crit'],$iface['inErr'],$iface['outErr']);
    }
    else
    {
        $str.= "NO GRAPH: Too many values for graphs, you have to choose one disk with -m; --match option\n";
    }
    echo trim($str)."\n";
    exit($global_state);
}
ssh_shell_close($shell);

function bitsToHumanStr($bits)
{
    $unitList = array('b/s','Kb/s','Mb/s','Gb/s');
    foreach($unitList as $unit)
    {
        if( $bits < 1000 )
        {
            return "$bits $unit";
        }
        else
        {
            $bits = round($bits/1000,3);
        }
    }
    return "$bits $unit";
}

function check_df($output,$disk_select,$warning,$critical,$rtrim,$unit,$divide)
{
    array_shift($output); //On enleve le header
    $disks = array();

    $global_state = 'OK';
    $str = "";
    foreach($output as $row)
    {
        $exp = explode(' ',$row);
        $ret_row = array();

        $ret_row['mounted'] = $exp[5];
        if( !empty($disk_select) && !preg_match("#$disk_select#",$ret_row['mounted']))
        {
            continue;
        }

        $ret_row['filesystem'] = $exp[0];
        $ret_row['used'] = rtrim($exp[2],$rtrim);
        $ret_row['avail'] = rtrim($exp[3],$rtrim);
        $ret_row['total'] = $ret_row['used'] + $ret_row['avail'];
        $ret_row['percent'] = rtrim($exp[4],'%');
        $disks[] = $ret_row;

        $percent = $ret_row['percent'];
        $used = round($ret_row['used']/$divide);
        $total = round($ret_row['total']/$divide);
        $mounted = $ret_row['mounted'];

        $result = calcul_max_state($warning,$critical,$percent,'%');
        $global_state = calcul_state($global_state,$result['state']);
        $str .= $result['state']. ": $mounted: $percent%used(${used}${unit}/${total}${unit})".$result['reason']."\n";
    }
    if( count($disks) == 0)
    {
        exit_state('UNKNOWN',"Mountpoint $disk_select not FOUND");
    }
    elseif(count($disks) == 1 )
    {
        $disk = array_shift($disks);
        $str.= sprintf("|used=%d;usedByte=%d;totalByte=%d",$disk['percent'],$disk['used'],$disk['total']);
    }
    else
    {
        $str.= "NO GRAPH: Too many values for graphs, you have to choose one disk with -m; --match option\n";
    }
    echo $str;
    exit($global_state);
}

function validate_treshold_calcul_min($warning,$critical,$min_treshold=NULL)
{
    if( $warning < $critical )
    {
        exit_state('UNKNOWN',"Warning treshold $warning must be lower than critical treshold $critical");
    }
    if( $min_treshold != NULL && $warning < $min_treshold )
    {
        exit_state('UNKNOWN',"Tresholds must be lowest than $min_treshold");
    }
}

function validate_treshold_calcul_max($warning,$critical,$max_treshold=NULL)
{
    if( $warning > $critical )
    {
        exit_state('UNKNOWN',"Warning treshold $warning must be bigger than critical treshold $critical");
    }
    if( $max_treshold != NULL && $critical > $max_treshold )
    {
        exit_state('UNKNOWN',"Tresholds must be lowest than $max_treshold");
    }
}

function calcul_min_state($warning,$critical,$value,$unit='')
{
    global $INT_STATUS;

    $ret = array();
    if( $value <= $critical )
    {
        $ret['state'] = 'CRITICAL';
        $ret['reason'] = " < $critical".$unit;
    }
    else if( $value <= $warning )
    {
        $ret['state'] = 'WARNING';
        $ret['reason'] = " < $warning".$unit;
    }
    else
    {
        $ret['state'] = 'OK';
        $ret['reason'] = " > $warning".$unit;
    }
    return $ret;
}

function calcul_max_state($warning,$critical,$value,$unit='')
{
    global $INT_STATUS;

    $ret = array();
    if( $value >= $critical )
    {
        $ret['state'] = 'CRITICAL';
        $ret['reason'] = " > $critical".$unit;
    }
    else if( $value >= $warning )
    {
        $ret['state'] = 'WARNING';
        $ret['reason'] = " > $warning".$unit;
    }
    else
    {
        $ret['state'] = 'OK';
        $ret['reason'] = " < $warning".$unit;
    }
    return $ret;
}

function secondsToPeriod($seconds)
{
    $seconds = intval($seconds);
    $dtF = new DateTime("@0");
    $dtT = new DateTime("@$seconds");
    return $dtF->diff($dtT)->format('%a days, %h hours, %i minutes and %s seconds');
}

function ssh_shell_open($host,$host_user,$host_pass)
{
    $port = 22;
    $exp = explode(':',$host);
    $addr = $exp[0];
    if(isset($exp[1]))
    {
        $port = $exp[1];
    }
    //$con=ssh2_connect($addr, $port,array('hostkey' => 'rsa-sha2-512,rsa-sha2-256,ecdsa-sha2-nistp256,ssh-ed25519,ssh-rsa,ssh-dss'));
    $con=@ssh2_connect($addr, $port);
    if( $con === FALSE )
    {
        throw new Exception("Unable to connect to $host:$port");
    }
    $ret = ssh2_auth_password($con, $host_user, $host_pass);
    if( $ret === FALSE )
    {
        throw new Exception("Unable to authenticate with user $host_user");
    }
    $shell=ssh2_shell($con, 'sh');
    if( $shell === FALSE )
    {
        throw new Exception("Unable to open interactive shell bash");
    }
    $ret = stream_set_blocking($shell, true);
    if( $ret === FALSE )
    {
        throw new Exception("Unable to set blocking stream");
    }
    return $shell;
}

function ssh_shell_multi_su($shell,$user_auth_list,$sleep=0.1)
{
    $seconds = floor($sleep);
    $nanoseconds = ($sleep - $seconds) * 1000000000;

    foreach($user_auth_list as $auth)
    {
        $user = $auth[0];
        $pass = $auth[1];
        fwrite( $shell, "su $user".PHP_EOL);
        time_nanosleep($seconds,$nanoseconds);
        if(!empty($pass))
        {
            fwrite( $shell, $pass.PHP_EOL);
            time_nanosleep($seconds,$nanoseconds);
        }
    }

    fwrite( $shell, 'echo SU_END_SU_END > /dev/null'.PHP_EOL);
    while($row=fgets($shell,4096))
    {
        flush();
        if( preg_match('/^su:.*/',$row) )
        {
            throw new Exception("Connection error : $row");
        }
        if ( strpos("$row","echo SU_END_SU_END")!== FALSE)
        {
            break;
        }
    }
}

function ssh_shell_clear($shell)
{
    fwrite( $shell, 'echo CLEAR_CLEAR'.PHP_EOL);
    while($row=fgets($shell,4096))
    {
        if ( strpos("$row",'echo CLEAR_CLEAR') === FALSE && strpos("$row","CLEAR_CLEAR")!== FALSE)
        {
            break;
        }
    }
}

function ssh_shell_oneline_cmd($shell,$cmd,$nb=1,$sleep=0.1)
{
    $seconds = floor($sleep);
    $nanoseconds = ($sleep - $seconds) * 1000000000;
    time_nanosleep($seconds,$nanoseconds);
    fwrite( $shell, $cmd .PHP_EOL);
    flush();

    //On recherche l'appel a la ligne de commande ci-dessus
    do
    {
        $row=fgets($shell,4096);
    }while(strpos("$row","$cmd") === FALSE);
    $row = '';
    //On attends puis on récupére la sortie
    time_nanosleep($seconds,$nanoseconds);
    for($i=0;$i<$nb;$i++)
    {
        $row.= fgets($shell,4096);
    }
    return $row;
}

function ssh_shell_read_file($shell,$file,$sleep=0.1)
{
    $seconds = floor($sleep);
    $nanoseconds = ($sleep - $seconds) * 1000000000;

    $row=trim(ssh_shell_oneline_cmd($shell,"wc -l $file",$sleep));
    $size = -1;
    if( preg_match('/^(\d+) \S+$/',$row,$matches))
    {
        $size = $matches[1];
    }
    if( $size != -1 )
    {
        fwrite( $shell,"cat $file" .PHP_EOL);
        time_nanosleep($seconds,$nanoseconds);
        flush();
        $output = '';
        for($i=0;$i<=$size;$i++)
        {
            $row=fgets($shell,4096);
            $output.=$row;
        }
        return $output;
    }
    else
    {
        return FALSE;
    }
}

function ssh_shell_cmd($shell,$cmd,$sleep=0.1)
{
    $seconds = floor($sleep);
    $nanoseconds = ($sleep - $seconds) * 1000000000;

    $file_name = basename(__FILE__);
    $file_name = uniqid($file_name).'.tmp';
    $file_name = "/tmp/$file_name";
    fwrite( $shell, "echo START${file_name}START > $file_name" .PHP_EOL);
    fwrite( $shell, "$cmd >> $file_name" .PHP_EOL);
    time_nanosleep($seconds,$nanoseconds);
    fwrite( $shell, "echo END${file_name}END >> $file_name" .PHP_EOL);
    fwrite( $shell, "cat $file_name" .PHP_EOL);

    $output = array();
    while($row=fgets($shell,4096)){
        flush();
        if ( strpos("$row","END${file_name}END")!== FALSE)
        {
            break;
        }
        $output[] = trim($row);
        if ( strpos("$row","START${file_name}START")!== FALSE)
        {
            $output = array(); // DEBUT : On efface le contenu précédent et on récupére la suite
        }
    }
    fwrite( $shell, "rm $file_name" .PHP_EOL);
    $row=fgets($shell,4096);
    return $output;
}

function ssh_shell_close($shell)
{
    ssh_shell_oneline_cmd($shell,"exit",1);
    //stream_set_timeout($shell, 2);
    ssh2_disconnect($shell);
}

function ssh_multi_su($host,$user_auth_list,$cmd,$sleep=0.1)
{
    $seconds = floor($sleep);
    $nanoseconds = ($sleep - $seconds) * 1000000000;

    $port = 22;
    $exp = explode(':',$host);
    $addr = $exp[0];
    if(isset($exp[1]))
    {
        $port = $exp[1];
    }

    $con=ssh2_connect($addr, $port);

    $first = array_shift($user_auth_list);
    $host_user = $first[0];
    $host_pass = $first[1];

    ssh2_auth_password($con, $host_user, $host_pass);
    $shell=ssh2_shell($con, 'bash');
    stream_set_blocking($shell, true);
    foreach($user_auth_list as $auth)
    {
        $user = $auth[0];
        $pass = $auth[1];
        fwrite( $shell, "su $user".PHP_EOL);
        time_nanosleep($seconds,$nanoseconds);
        if(!empty($pass))
        {
            fwrite( $shell, $pass.PHP_EOL);
            time_nanosleep($seconds,$nanoseconds);
        }
    }

    $file_name = basename(__FILE__);
    $file_name = uniqid($file_name).'.tmp';
    $file_name = "/tmp/$file_name";
    fwrite( $shell, "echo START${file_name}START > $file_name" .PHP_EOL);
    fwrite( $shell, "$cmd >> $file_name" .PHP_EOL);
    fwrite( $shell, "echo END${file_name}END >> $file_name" .PHP_EOL);
    fwrite( $shell, "cat $file_name" .PHP_EOL);

    $output = array();
    while($row=fgets($shell,4096)){
        flush();
        if( preg_match('/^su:.*/',$row) )
        {
            throw new Exception("Connection error : $row");
        }
        if ( strpos("$row","END${file_name}END")!== FALSE)
        {
            break;
        }
        $output[] = trim($row);
        if ( strpos("$row","START${file_name}START")!== FALSE)
        {
            $output = array(); // DEBUT : On efface le contenu précédent et on récupére la suite
        }
    }
    fwrite( $shell, "rm $file_name" .PHP_EOL);
    fclose($shell);
    return $output;
}

function exit_state($state,$msg=null,$usage=false){
        $STR_STATUS=$GLOBALS["STR_STATUS"];
        $INT_STATUS=$GLOBALS["INT_STATUS"];

        $ret=$state;
        if( is_string($state) ) $ret=$INT_STATUS[strtoupper($state)];
        else $state=$STR_STATUS[$ret];

        if( $msg != null )
        {
          $msg=trim($msg);
          echo "$state : $msg\n";
        }
        if( $usage ) print_usage(true);
        exit($ret);
}

function calcul_state($a,$b=null){
        $STR_STATUS=$GLOBALS["STR_STATUS"];
        $INT_STATUS=$GLOBALS["INT_STATUS"];
        $PRIORITIES_STATES=$GLOBALS["PRIORITIES_STATES"];

        if(is_string($a) ) $a=$INT_STATUS[strtoupper($a)];
        if(is_string($b) ) $b=$INT_STATUS[strtoupper($b)];

        foreach($PRIORITIES_STATES as $value){
                $value = $INT_STATUS[$value];

                if( $value == $a || $value == $b ) return $value;
        }
        return $INT_STATUS["UNKNOWN"];
}

class Parameters
{
    private $data = array();
    private $options = NULL;
    private $params = NULL;
    private $usage = array();

    function __construct()
    {
    }

    function addUsage($row)
    {
        $this->usage[] = $row;
    }

    static function getArrayBiggestWordSize($arr){
    	$biggest_size = 0;
            foreach($arr as $word){
                    $cur_size = strlen($word);
                    if( $biggest_size < $cur_size){
                            $biggest_size = $cur_size;
                    }
            }
    	return $biggest_size;
    }

    function print_usage($alone=TRUE)
    {
        $script = basename(__FILE__);
        $left = 'usage : ';
        $size_left = strlen($left);

        foreach($this->usage as $row)
        {
            echo str_pad($left,$size_left) . "$script $row [OPTIONS]\n";
            $left = '';
        }
        echo"\n";
        if($alone)
        {
            $row = "--help for more help";
            echo str_pad($left,$size_left) . "$script $row\n";
        }
    }

    function print_help()
    {
        $this->print_usage(FALSE);
        $opt = array();
        $desc = array();
        foreach($this->data as $key => $row)
        {
            $str = "\t-" . $this->getParamOptionsString($key);
            if( !empty($row['value']) )
            {
                $str.= '=<'.trim($row['value'],'<>').">";
            }
            $opt[] = $str;
            $desc[] = $row['descr'];
        }
        $size_left = self::getArrayBiggestWordSize($opt);
        foreach($opt as $idx => $left)
        {
            echo str_pad($left,$size_left) . "  ".$desc[$idx]."\n";
        }
    }

    public function getParamOptionsString($long_opt)
    {
        $row = $this->data[$long_opt];
        return $row['short'] . ", --" . $row['long'];
    }

    public function addOpt($optional,$short_opt,$long_opt,$value_desc,$descr,$default = NULL)
    {
        $row = array();
        $row['optional'] = $optional;
        $row['short'] = $short_opt;
        $row['long'] = $long_opt;
        $row['value'] = $value_desc;
        $row['descr'] = $descr;
        $row['default'] = $default;
        $this->data[$long_opt] = $row;
    }

    private function getopt()
    {
        if( $this->options != NULL )
        {
            return;
        }

        $short_opts = '';
        $long_opts = array();

        foreach($this->data as $row)
        {
            $short_opts.= $row['short'].$row['optional'];
            $long_opts[] = $row['long'].$row['optional'];
        }
        $this->options = getopt($short_opts, $long_opts);
        $this->params = array();
        foreach($this->data as $row)
        {
            $req_opt = $row['optional'];
            $long_opt = $row['long'];
            $short_opt =  $row['short'];
            $default_opt = $row['default'];
            $key_name = $row['long'];

            if( $req_opt == '' )
            {
                if( isset($this->options[$long_opt]) || isset($this->options[$short_opt] ) )
                {
                    $value = TRUE;
                }
                else
                {
                    $value = FALSE;
                }
            }
            elseif( isset($this->options[$long_opt]))
            {
                $value = $this->options[$long_opt];
            }
            elseif( isset($this->options[$short_opt] ))
            {
                $value = $this->options[$short_opt];
            }
            else
            {
                continue;
                $value = $default_opt;
            }
            $this->params[$key_name] = $value;
        }
    }

    public function getParam($name,$default=NULL)
    {
        $this->getopt();
        if( $default != NULL && empty($this->params[$name]) )
        {
            return $default;
        }
        elseif( !isset($this->params[$name]))
        {
            return NULL;
        }
        elseif(empty($this->params[$name]))
        {
            return FALSE;
        }
        return $this->params[$name];
    }
}
?>
